/*
Altered Dawnville level, written by John Haggerty

Structure of this script:

Each of the following sections are portions of the game which, when finished, advance to the next portion:
 - main() and related functions
 - FIGHT 1: from start to where the germans pull back
 - FIGHT 2: small battle where the mortars appear and foley tells you to go with him
 - CHURCH RUN: movement to the church with foley and baker, including the small fight before entering
 - GRAVEYARD RUN: movement across the graveyard up to the point where the graveyard axis are dead
 - MORTAR BATTLE: fight during which the mortar teams are taken out, to the point where foley kicks open the door
 - FIGHT 3: where the player must use the MG42
 - TANK 1: first tank appears, player takes it out
 - BACK DOOR: battle with the germans in the back of the house
 - TANK 2: second tank appears, player takes it out
 - REINFORCEMENTS: timer until reinforcements appear, end of game.

The rest of the file consists of various (groups of) functions that are used many times:
 - Spawn functions: all of the functions that allow control of spawning AI. This is done through a "spawn ID" system,
     where each AI is given an ID when spawning, and groups of AI are manipulated through functions by passing this ID.
 - Tank stuff: functions that handle tank explosions and panzerfausts
 - Mortar stuff: functions related to dropping mortars
 - Animation stuff: functions that initialize and control character animations and effects
 - Helpful functions: various functions that didn't fit anywhere else
 - Event handling functions: a simple interface to the notify/waittill commands, but keeps track of events that have
     happened in the past. Similar to flag_wait and flag_set type functions in dawnville.gsc.

*/

#using_animtree("generic_human");

main()
{
	precacheShellshock("default");
	
	precacheItem("panzerfaust");
	precacheModel("xmodel/ammo_panzerfaust_box2");
	precacheModel("xmodel/weapon_panzerfaust");
	precacheModel("xmodel/vehicle_tank_panzeriv_machinegun");

	precacheString(&"MYDAWNVILLE_TIMER_TEXT");
	
	maps\_load::main();
	maps\_mortarteam::main();
	maps\_treefall::main();
	maps\_tank::main();
	maps\_tiger::main_camo();
	
	level.ambient_track ["exterior"] = "ambient_dawnville";
	level.ambient_track ["interior"] = "ambient_dawnville_int";
	thread maps\_utility::set_ambient("exterior");

	level.player takeallweapons();
	level.player giveWeapon("m1carbine");
	level.player giveWeapon("thompson");
	level.player giveWeapon("colt");
	level.player giveWeapon("fraggrenade");
	level.player switchToWeapon("thompson");
	
	// these variables let you skip some parts
	/*level.introquick = true;
	level.fight1quick = true;
	level.fight2quick = true;
	level.churchrunquick = true;
	level.graveyardquick = true;
	level.mortarquick = true;
	level.fight3quick = true;
	level.tank1quick = true;
	level.backdoorquick = true;
	level.tank2quick = true;
	level.reinforcementsquick = true;*/
	
	initEvents();
	initSpawns();
	initCharacters();
	initMortars();
	initFX();
	initTankExplosions();
	initPanzerfaust();
	
	exploderChunkFix();
	exploderLightingFix();
	
	thread intro();
	
	wait(.01);

	// begin gameplay
	thread fight1();
	
	
	/*
	// TEMPORARY for fast testing (skips everything before church, must comment thread fight1() above)
	level.mortarMinInterval = 2;
	level.mortarMaxInterval = 7;
	level.mortarTargetsPlayer = false;
		
	thread mortars();

	level.player SetFriendlyChain(getnode("allied advance node chain", "targetname"));
	allies = getaiarray("allies");
	for (i = 0; i < allies.size; i++)
		allies[i] setgoalentity (level.player);

	thread church_run();
	triggeredEvent("following foley 1", "entered following foley 1");*/
}


intro()
{
	if (isdefined(level.introquick) && level.introquick) {
		wait(.1);
		eventOccur("Intro screen start fade in");
		return;
	}

	// not using introscreen function because i don't want text and i want more control over fadein
	
	level.introblack = newHudElem();
	level.introblack.x = 0;
	level.introblack.y = 0;
	level.introblack setShader("black", 640, 480);

	level.player shellshock("default", 3.25);

	wait (1);

	level.player allowStand(false);
	level.player allowCrouch(true);
	level.player allowProne(false);

	wait (1.5);
	
	eventOccur("Intro screen start fade in");
	
	level.player allowStand(true);
	level.player allowProne(true);

	level.introblack fadeOverTime(3.5); 
	level.introblack.alpha = 0;
	
	wait (6);
	
	level.introblack destroy();
	
	
}

// -------------------------------------
//  FIGHT 1
// -------------------------------------

fight1()
{
	// set up objective
	objective_add(0, "active", &"MYDAWNVILLE_OBJ_PUSH_BACK", getent("objective0 location", "targetname").origin);
	objective_current(0);

	setMaxSpawn("spawn ID fight1 axis", 12);

	// spawn a few axis players to start off
	quickSpawn("start fight axis spawn", "spawn ID fight1 axis", "stop fight1 spawning", 6);
	
	waitEvent("Intro screen start fade in");
	wait(1.5);
	
	// spawn a few axis players where they'll cause an immediate fight, to ensure some gunfire when game starts
	immediatespawn = getentarray("immediate axis spawn", "targetname");
	for (i = 0; i < immediatespawn.size; i++)
		thread forceSpawn(immediatespawn[i], "spawn ID fight1 axis", "stop fight1 spawning");

	// spawn axis players until either x have been spawned or y seconds have passed,
	// whichever comes first
	
	thread continuousSpawn(
		"start fight axis spawn", // spawners targetname
		"spawn ID fight1 axis", // spawn ID for the guys spawned
		"stop fight1 spawning", // stop spawning if this event happens
		"fight1 axis spawn finished", // throw this event when finished
		2, 5, // min - max interval between spawns
		8, // number to spawn
		6 // try to keep at least this many alive at once
	);
	
	if (isdefined(level.fight1quick) && level.fight1quick)
		timedEvent(3, "fight1 timer");
	else
		timedEvent(20, "fight1 timer");
	
	// get a little yelling going
	wait(.2);

	level.baker thread anim_single_solo (level.baker, "here they come");
	
	wait(1.5);
	
	level.foley thread anim_single_solo (level.foley, "keep heads down");
	
	
	// wait until fight has gone on for a bit
	waitAnyEvent("fight1 axis spawn finished", "fight1 timer");
	eventOccur("stop fight1 spawning");
	
	
	// make axis run
	axis = getGuysBySpawnID("spawn ID fight1 axis");
	axisrunnodes = getNodeArrayFromChain("axis run to");
	for (i = 0; i < axis.size; i++) {
		axis[i] thread doRetreat(axisrunnodes[randomint(axisrunnodes.size)]);
	}
	
	wait(3);
	
	level.foley thread anim_single_solo (level.foley, "push them back");
	
	wait(1.5);
	
	thread fight2();
}

// -------------------------------------
//  FIGHT 2
// -------------------------------------

fight2()
{
	objective_position(0, getent("objective0 location2", "targetname").origin);

	// allies push on
	level.player SetFriendlyChain(getnode("allied advance node chain", "targetname"));
	allies = getaiarray("allies");
	for (i = 0; i < allies.size; i++)
		allies[i] setgoalentity (level.player);
		
	wait(7);

	// begin infinite spawn of axis beyond where the player can reach
	setMaxSpawn("spawn ID fight2 axis", 8);
	thread continuousSpawn(
		"fight2 axis spawn", // spawners targetname
		"spawn ID fight2 axis", // spawn ID for the guys spawned
		"stop fight2 spawning", // stop spawning if this event happens
		"fight2 axis spawn finished", // throw this event when finished
		2, 5, // min - max interval between spawns
		-1, // number to spawn (-1 = infinite)
		5 // try to keep at least this many alive at once
	);

	// wait until the player has fought for 15 seconds straight
	fightarea = getent("fight2 area", "targetname");
	timecount = 0;
	if (isdefined(level.fight2quick) && level.fight2quick)
		timerlen = 1;
	else
		timerlen = 15;
	while (1)
	{
		if (level.player istouching(fightarea))
			timecount++;
		else
			timecount = 0;
		
		if (timecount >= timerlen)
			break;
		
		wait(1);
	}

	// any point after this the player may start following foley (even though he hasn't started moving yet)
	// start monitoring whether they do now, so that we don't miss it if they go early
	triggeredEvent("following foley 1", "entered following foley 1");
	
	// it's mortar time
	doMortarNearPlayer();
	wait(.4);
	level.foley thread anim_single_solo (level.foley, "mortars");
	wait(.7);
	doMortarNearPlayer();
	wait(.3);
	level.baker thread anim_single_solo (level.baker, "mortars incoming");

	wait(2);
	doAnyMortar();
	
	wait(1);
	
	level.mortarMinInterval = 2;
	level.mortarMaxInterval = 7;
	level.mortarTargetsPlayer = false;
	
	thread mortars();
	
	thread church_run();
}

mortars()
{
	level endon("all mortars dead");
	while (1)
	{
		wait(level.mortarMinInterval + randomfloat(level.mortarMaxInterval - level.mortarMinInterval));
		if (level.mortarTargetsPlayer && randomfloat(1) < .5)
			thread doMortarVeryNearPlayer();
		else
			thread doAnyMortar();
	}
}

// -------------------------------------
//  CHURCH RUN
// -------------------------------------
church_run()
{
	objective_state(0, "done");
	
	objective_add(1, "active", &"MYDAWNVILLE_OBJ_THROUGH_CHURCH", getent("church objective location", "targetname").origin);
	objective_current(1);
	
	/*if (isdefined(level.churchrunquick) && level.churchrunquick)
	{
		maps\_utility::exploder(100);
	}*/
	
	foleynode = getnode("foley to church 1", "targetname");
	bakernode = getnode("baker to church 1", "targetname");
	
	level.foley.suppressionwait = 0;
	
	level.foley setGoalNode(foleynode);
	level.baker setGoalNode(bakernode);
	level.foley.goalradius = 4;
	level.baker.goalradius = 50;
	level.foley thread anim_single_solo (level.foley, "to church with me");
	
	level.foley waittill("goal");
	level.baker.suppressionwait = 0;
	level.foley lookat(level.player, 10);
	
	// wait until player follows (or even goes on ahead)
	//thread mortarTargetsPlayerAfterTime(20);
	thread tellPlayerToFollowFoley();
	waitEvent("entered following foley 1");
	
	level notify("don't punish player");
	level.mortarTargetsPlayer = false;
	
	level.mortarMinInterval = 3;
	level.mortarMaxInterval = 9;
	
	thread mortarScare();
	
	
	// moving on to the church
	level.foley.goalradius = 32;
	level.baker.goalradius = 32;
	level.foley setGoalNode(getnode("foley to church 2", "targetname"));
	level.baker setGoalNode(getnode("baker to church 2", "targetname"));

	followingtrigger = getent("following foley 2", "targetname");
	followingtrigger waittill("trigger");
	
	level.foley.goalradius = 512;
	level.baker.goalradius = 512;
	
	autoSave(1);
	
	level.foley.suppressionwait = 5;
	level.baker.suppressionwait = 5;
	
	if (isdefined(level.churchrunquick) && level.churchrunquick)
	{
		eventOccur("church run axis spawning complete");
		eventOccur("church run axis are all dead");
	}
	else
	{
		// small fight outside church
		// first spawn one guy
		forceSpawn(getent("church run axis spawn first", "targetname"), "spawn ID church run");

		wait (.5);
		// warning from baker about impending fight
		level.baker thread anim_single_solo (level.baker, "incoming take cover");

		// spawn some more
		thread jitterSpawn("church run axis spawn", "spawn ID church run", 1, 3, "entered church", "church run axis spawning complete");
		// throw an event when they've all died
		spawnFullDeathEvent("spawn ID church run", "church run axis are all dead", "church run axis spawning complete");

		// wait for some of the guys we just spawned to die
		spawnDeathEvent("spawn ID church run", 5, "church run axis guys died");
		waitEvent("church run axis guys died");
	}
	
	// wait until player looks at wall that will explode, or until 10 seconds have passed
	triggeredEvent("mortar church explode lookat", "player looked at church explosion wall");
	timedEvent(10, "church explosion timer");
	waitAnyEvent("player looked at church explosion wall", "church explosion timer");
	
	// get baker and foley closer to where we want them
	level.foley.goalradius = 4;
	level.baker.goalradius = 4;
	
	// spawn a guy in the church just for fun
	thread forceSpawn(getent("church axis spawn", "targetname"), "spawn ID inside church", "entered church", "inside church guy spawned");
	spawnFullDeathEvent("spawn ID inside church", "inside church guy is dead", "inside church guy spawned", "graveyard run started");
	
	thread waitOnPlayerChurchEntrance();

	// explode church wall
	//if (!isdefined(level.churchrunquick) || !level.churchrunquick)
	{
		doMortar(getent("mortar church explode spot", "targetname"));
		wait(.2);
		maps\_utility::exploder(100);
	}
	
	// now that the church wall is opened, the player could theoretically get to the mortar battle area at any time
	thread mortar_battle();

	// start killing off the guys we spawned so that the player doesn't have to wait an hour for one of them to poke their head up
	thread killOffSlowly(getGuysBySpawnID("spawn ID church run"), 4,6);
	
	// wait for all of the guys we spawned to die before we send baker and foley on their way
	waitEvent("church run axis are all dead");
	
	level.foley thread anim_single_solo (level.foley, "the church go go go");
	
	level.foley setGoalNode(getnode("foley church destination", "targetname"));
	level.baker setGoalNode(getnode("baker church destination", "targetname"));

	waitEvent("entered church");
	
	thread graveyard_run();
}

tellPlayerToFollowFoley()
{
	if (hasEventOccured("entered following foley 1"))
		return;
	level endon("entered following foley 1");
	
	while(1)
	{
		wait(8 + randomfloat(6));
		level.foley thread anim_single_solo (level.foley, "to church with me short");
	}
}

waitOnPlayerChurchEntrance()
{
	// wait for player to enter church
	churchEntrance = getent("church entrance", "targetname");
	churchEntrance waittill("trigger");

	// in the meantime, fight 2 is still going on; scale it down some
	setMaxSpawn("spawn ID fight2 axis", 5);
	
	// sic remaining enemies from church run fight onto the player
	remainingguys = getGuysBySpawnID("spawn ID church run");
	for (i = 0; i < remainingguys.size; i++) {
		remainingguys[i] setGoalEntity(level.player);
		remainingguys[i].goalradius = 4;
	}
	
	eventOccur("entered church");
}

mortarScare()
{
	// make a couple of mortars land in front of the player as they head to the church
	trigger = getent("mortar scare 1", "targetname");
	trigger waittill("trigger");
	
	thread doMortarRelativeToPlayer((randomfloat(1)*2-1, 5, 0), 200, .01);

	//trigger = getent("mortar scare 2", "targetname");
	//trigger waittill("trigger");
	
	//thread doMortarRelativeToPlayer((randomfloat(1)*2-1, 5, 0), 220, .01);
}

/*mortarTargetsPlayerAfterTime(duration) // this was evil.
{
	level endon("don't punish player");
	wait(duration);
	level.mortarTargetsPlayer = true;
}*/

// -------------------------------------
//  GRAVEYARD RUN
// -------------------------------------

graveyard_run()
{
	// wait until the guy in the church is dead, or until some time passes, or until the player tries to leave the church
	timedEvent(10, "graveyard run start timer");
	triggeredEvent("church exit area", "at church exit");
	
	waitAnyEvent("at church exit", "graveyard run start timer", "inside church guy is dead");
	eventOccur("graveyard run started");
	
	// make all the people in the main fight ignored so that the axis near the player don't look at them through the walls
	allies = getaiarray("allies");
	for (i = 0; i < allies.size; i++) {
		if (allies[i] != level.baker && allies[i] != level.foley) {
			thread maintainIgnoreMe(allies[i], "stop ignoring allies");
		}
	}
	
	// remove monster clip at church door
	monclip = getent("church exit monster clip", "targetname");
	monclip notsolid();
	monclip connectpaths();
	monclip delete();

	if (isdefined(level.graveyardquick) && level.graveyardquick)
	{
		objective_state(1, "done");
		objective_add(2, "active", &"MYDAWNVILLE_OBJ_COVER_FOLEY", getent("church objective location", "targetname").origin);
		objective_state(2, "done");
		objective_add(3, "active", &"MYDAWNVILLE_OBJ_CROSS_GRAVEYARD", getent("objective inside house", "targetname").origin);
		objective_state(3, "done");
		objective_add(4, "active", &"MYDAWNVILLE_OBJ_COVER_BAKER", getent("objective inside house", "targetname").origin);
		objective_state(4, "done");
		objective_add(5, "active", &"MYDAWNVILLE_OBJ_FINISH_GRAVEYARD", getent("graveyard finish area", "targetname").origin);
		objective_current(5);
		
		monclip = getent("graveyard monster clip", "targetname");
		monclip notsolid();
		monclip connectpaths();
		monclip delete();

		level.foley setGoalNode(getnode("foley graveyard finish", "targetname"));
		level.baker setGoalNode(getnode("baker graveyard finish", "targetname"));

		eventOccur("all crossed graveyard");
		//eventOccur("graveyard finished"); // we'll do this later
		
		return;
	}

	// finish previous objective
	objective_state(1, "done");

	// this is for later, but we start monitoring now since it could happen at any time
	triggeredEvent("inside house", "player entered house");
	
	level.graveyardState = "in church";
	
	level.player.oldthreatbias = level.player.threatbias;
	level.foley.oldthreatbias = level.foley.threatbias;
	level.baker.oldthreatbias = level.baker.threatbias;
	
	level.player.threatbias = 0;
	level.foley.threatbias = 0;
	level.baker.threatbias = 0;
	
	// make it unlikely foley or baker will kill anyone
	level.foley.oldaccuracy = level.foley.accuracy;
	level.baker.oldaccuracy = level.baker.accuracy;

	level.foley.accuracy = 0;
	level.baker.accuracy = 0;

	// start graveyard axis spawning
	setMaxSpawn("spawn ID graveyard axis", 6);
	setSpawnIgnoreMe("spawn ID graveyard axis", true);
	thread continuousSpawn(
		"graveyard axis spawn", // spawners targetname
		"spawn ID graveyard axis", // spawn ID for the guys spawned
		"all crossed graveyard", // stop spawning if this event happens
		"", // throw this event when finished
		2, 5, // min - max interval between spawns
		-1, // number to spawn (-1 = inf)
		3 // try to keep at least this many alive at once
	);
	
	thread maintainIgnoreMe(level.foley, "stop ignoring foley");
	thread maintainIgnoreMe(level.baker, "stop ignoring baker");
	
	level.protectWithIgnoring = true;
	thread protectPlayerWhileInChurchOrHouse();

	if (!hasEventOccured("at church exit"))
		wait(1.5); // don't let foley speak over the death sound of the axis church guy

	// foley gives speech
	level.foley waittill("goal");
	thread foleyLookAtBakerAndPlayer();
	level.foley thread anim_single_solo (level.foley, "graveyard speech");
	level.foley playsound("mydawnville_foley_graveyardspeechstart");
	//level.foley thread anim_loop_solo(level.foley, "idle", undefined, "graveyard speech over");
	level.foley waittill("graveyard speech");
	eventOccur("graveyard speech over");
	
	// set up new objective
	objective_add(2, "active", &"MYDAWNVILLE_OBJ_COVER_FOLEY", getent("church objective location", "targetname").origin);
	objective_current(2);
	
	level.protectWithIgnoring = false;
	
	autoSave(2);
	
	// keep a guy on the MG42 frequently
	thread graveyardMG42("all crossed graveyard");
	wait(1);

	level notify("stop ignoring foley"); level.foley.ignoreme = false;
	level notify("stop ignoring baker"); level.baker.ignoreme = false;
	setSpawnIgnoreMe("spawn ID graveyard axis", false);

	// get foley ready to run
	level.foley setGoalNode(getnode("church exit wait", "targetname"));
	// put baker in position to cover him
	level.baker setGoalNode(getnode("baker church cover", "targetname"));
	
	// FOLEY RUNS FOR IT ------------------------------------
	
	// wait until foley is ready
	level.foley waittill("goal");
	
	//level notify("stop ignoring foley");
	//level.foley.ignoreme = false;
	level.foley.threatbias = 9999999;
	
	level.foley endon("death");
	thread missionFailFoleyDies();
	
	// let foley wait a bit
	wait(5);
	
	debug("FOLEY GOING");
	
	// another reminder to cover foley
	level.foley thread anim_single_solo (level.foley, "keep shooting");
	
	level.foley.suppressionwait = 0;
	setGraveyardMG42NonPlayerTarget(level.foley);

	// start requiring that enemies die occasionally
	// (requiring at least one enemy death every 5 seconds)
	thread protectBasedOnDeathRate("foley graveyard protection", level.foley, "spawn ID graveyard axis", 5, "stop foley graveyard protection");

	// send foley halfway
	level.foley allowedstances("prone", "crouch");
	level.foley setGoalNode(getnode("graveyard midway", "targetname"));
	
	// put baker in position to go when he's ready
	level.baker setGoalNode(getnode("church exit wait", "targetname"));
	
	level.foley waittill("goal");
	// let him wait a bit
	wait(3);
	// send foley the rest of the way
	level.foley setGoalNode(getnode("foley house wait", "targetname"));
	level.foley waittill("goal");
	
	// foley made it!
	//thread maintainIgnoreMe(level.foley, "stop ignoring foley");
	level.foley.threatbias = 0;
	unsetGraveyardMG42NonPlayerTarget();
	
	eventOccur("stop foley graveyard protection");
	resetMagicBulletShield(level.foley);
	
	level.foley allowedstances("prone", "crouch", "stand");
	level.foley.suppressionwait = 5;
	
	level notify("foley survives graveyard");

	thread moveFoleyWhenPlayerArrivesFromGraveyard("all crossed graveyard");
	
	// PLAYER RUNS FOR IT ------------------------------------

	objective_state(2, "done");
	objective_add(3, "active", &"MYDAWNVILLE_OBJ_CROSS_GRAVEYARD", getent("objective inside house", "targetname").origin);
	objective_current(3);
	
	level.foley.accuracy = level.foley.oldaccuracy;
	level.baker.accuracy = level.baker.oldaccuracy;

	level.graveyardState = "in graveyard";

	// if player is already at or past house (they cheated and went early), don't do the preparation for him to get there
	if (!hasEventOccured("player entered house") && !hasEventOccured("player left graveyard"))
	{
		// explanation from baker that you must go now
		level.baker thread anim_single_solo (level.baker, "martin you're next");
		level.baker playsound("mydawnville_baker_martinyourenextstart");

		level.baker waittill("martin you're next");
		wait(2);
		
		level.baker.oldaccuracy = level.baker.accuracy;
		level.baker.oldbravery = level.baker.bravery;
		level.baker.bravery = 500000;
		level.baker.accuracy = 1000;
		thread maintainIgnoreMe(level.baker, "stop ignoring baker");
		
		// put baker on a path node instead of a cover node so he doesn't think he has cover (which seems to make him sit there helpless)
		level.baker setGoalNode(getnode("baker shoot mg42 from", "targetname"));
		level.baker allowedStances("stand"); // stand up, you idiot
		
		thread bakerKillMG42();
		waitEvent("player entered house");
		
		level.baker.accuracy = level.baker.oldaccuracy;
		level.baker.bravery = level.baker.oldbravery;
		level notify("stop ignoring baker");
		level.baker.ignoreme = false;
		level.baker setGoalNode(getnode("church exit wait", "targetname"));
		level.baker allowedstances("prone", "crouch", "stand");
		
		level.foley thread anim_single_solo (level.foley, "good job son");
	}

	level.foley.accuracy = 0;
	level.baker.accuracy = 0;

	level.graveyardState = "in house";

	// BAKER RUNS FOR IT ------------------------------------

	objective_state(3, "done");
	objective_add(4, "active", &"MYDAWNVILLE_OBJ_COVER_BAKER", getent("objective inside house", "targetname").origin);
	objective_current(4);
	
	//level notify("stop ignoring baker");
	//level.baker.ignoreme = false;
	level.baker.threatbias = 9999999;
	level.baker endon("death");
	thread missionFailBakerDies();
	
	// let baker wait a bit
	wait(4.5);
	level.foley thread anim_single_solo (level.foley, "covering fire");
	wait(2.5);

	debug("BAKER GOING");
	level.baker.suppressionwait = 0;
	setGraveyardMG42NonPlayerTarget(level.baker);

	// start requiring that enemies die occasionally
	// (requiring at least one enemy death every 4 seconds)
	thread protectBasedOnDeathRate("baker graveyard protection", level.baker, "spawn ID graveyard axis", 4, "stop baker graveyard protection");
	
	// send baker halfway
	level.baker allowedstances("prone", "crouch");
	level.baker setGoalNode(getnode("graveyard midway", "targetname"));
	
	level.baker thread anim_single_solo (level.baker, "charge");
	
	level.baker waittill("goal");
	// let him wait a bit
	wait(3);
	// send baker the rest of the way
	level.baker setGoalNode(getnode("baker house defend", "targetname"));
	level.baker waittill("goal");
	
	// baker made it!
	//thread maintainIgnoreMe(level.baker, "stop ignoring baker");
	//level.baker.threatbias = 0;
	eventOccur("stop baker graveyard protection");
	resetMagicBulletShield(level.baker);
	
	level.baker allowedstances("prone", "crouch", "stand");
	level.baker.suppressionwait = 5;
	unsetGraveyardMG42NonPlayerTarget();

	level notify("baker survives graveyard");
	
	level.foley.accuracy = level.foley.oldaccuracy;
	level.baker.accuracy = level.baker.oldaccuracy;

	// ALL THREE MADE IT ------------------------------------
	
	eventOccur("all crossed graveyard");

	level notify("stop protecting player while in church or house");
	//level.player.ignoreme = false;
	//level notify("stop ignoring foley");
	//level.foley.ignoreme = false;
	//level notify("stop ignoring baker");
	//level.baker.ignoreme = false;
	level.player.threatbias = level.player.oldthreatbias;
	level.foley.threatbias = level.foley.oldthreatbias;
	level.baker.threatbias = level.baker.oldthreatbias;
	
	// make the MG42 auto-fire
	mg42 = getent("graveyard mg42", "targetname");
	mg42 setmode("auto_ai");
	
	// remove graveyard monster clip
	monclip = getent("graveyard monster clip", "targetname");
	monclip notsolid();
	monclip connectpaths();
	monclip delete();
	
	// finish up the guys in the graveyard
	
	level.foley thread anim_single_solo (level.foley, "okay move move");
	
	objective_state(4, "done");
	
	objective_add(5, "active", &"MYDAWNVILLE_OBJ_FINISH_GRAVEYARD", getent("graveyard finish area", "targetname").origin);
	objective_current(5);
	
	level.foley setGoalNode(getnode("foley graveyard finish", "targetname"));
	level.baker setGoalNode(getnode("baker graveyard finish", "targetname"));
	
	// wait until they're dead
	spawnFullDeathEvent("spawn ID graveyard axis", "graveyard axis are all dead");
	waitEvent("graveyard axis are all dead");
	
	eventOccur("graveyard finished");
}

foleyLookAtBakerAndPlayer()
{
	level endon("graveyard speech over");
	
	while(1)
	{
		time = 2 + randomfloat(2);
		level.foley lookat(level.player, time);
		wait(time);
		
		time = 2 + randomfloat(2);
		level.foley lookat(level.baker, time);
		wait(time);
	}
}

// this is necessary to circumvent magic_bullet_shield's resetting of ignoreme
maintainIgnoreMe(guy, stopEvent)
{
	level endon(stopEvent);
	while(1)
	{
		guy.ignoreme = true;
		wait(.25);
	}
}

bakerKillMG42()
{
	if (hasEventOccured("player entered house"))
		return;
	level endon("player entered house");
	
	thread bakerSayGoGoGo();
	
	while (1)
	{
		if (!level.graveyardMG42GuyAlive)
			waitEvent("graveyard mg42 spawn");

		// make baker kill mg42 guy
		level.graveyardMG42Guy.threatbias = 99999999;

		waitEvent("graveyard mg42 spawn");

		// wait a bit before baker tries to kill the mg42 guy again
		timedEventOnce(5 + randomfloat(5), "ready to rekill graveyard mg42");
		waitEvent("ready to rekill graveyard mg42");
	}
}
bakerSayGoGoGo()
{
	if (hasEventOccured("player entered house"))
		return;
	level endon("player entered house");

	while(1)
	{
		waitEvent("graveyard mg42 death");

		if (level.player istouching(getent("church main area", "targetname")) || level.player istouching(getent("church exit area", "targetname")))
		{
			wait(.3);
			level.baker thread anim_single_solo (level.baker, "go go go");
		}
	}
}

// the following functions all have to do with getting the MG42 in the graveyard to behave
// somewhat intelligently, targetting the guy who crosses, and targetting air inbetween 
// when no one's crossing.
// it also keeps a guy on the MG42 as often as possible.
// (auto fire wasn't working because the turret guy left the turret when he had nothing to shoot at =( )
graveyardMG42(stopEvent)
{
	level endon(stopEvent);
	
	level.graveyardMG42Inited = true;
	
	level.graveyardMG42GuyAlive = false;
	level.graveyardMG42TargettingGuy = false;
	level.graveyardMG42Active = false;
	
	level.graveyardMG42HasNonPlayerTarget = false;
	level.graveyardMG42TargettingPlayer = false;

	spawner = getent("graveyard axis mg42 spawn", "targetname");
	while(1)
	{
		// temporarily use a different spawn ID so we're not affected by the maxspawn limit
		forceSpawn(spawner, "spawn ID graveyard mg42", stopEvent);
		
		level.graveyardMG42Guy = level.lastSpawn;
		level.graveyardMG42GuyAlive = true;
		switchSpawnID(level.graveyardMG42Guy, "spawn ID graveyard axis");
		level.graveyardMG42Guy.accuracy = 1;
		
		thread graveyardMG42RandomTarget(stopEvent);
		thread setGraveyardMG42Active(level.graveyardMG42Guy);
		thread graveyardMG42KeepGuyOnTurret(level.graveyardMG42Guy);
		
		// notify the mg42 guy has spawned
		level notify("graveyard mg42 spawn");
		
		level.graveyardMG42Guy waittill("death");
		
		// and died
		level notify("graveyard mg42 death");
		level.graveyardMG42GuyAlive = false;
		level.graveyardMG42Active = false;
		
		// wait a short time before spawning another one
		wait(1 + randomfloat(3));
	}
}

// even now, sometimes the guy just randomly leaves the turret.
// nothing to do but put him back on it =(
graveyardMG42KeepGuyOnTurret(guy)
{
	guy endon("death");
	level endon("all crossed graveyard");

	mg42 = getent("graveyard mg42", "targetname");
	while(1)
	{
		if (distance(guy.origin, mg42.origin) > 32) {
			node = getnode(getent("graveyard axis mg42 spawn", "targetname").target, "targetname");
			guy setGoalNode(node);
			guy waittill("goal");
		}
		guy useturret(mg42);
		
		wait(.4);
	}
}

setGraveyardMG42Active(guy)
{
	guy waittill("goal");
	wait(.01);
	if (isalive(guy)) {
		level.graveyardMG42Active = true;
		level notify("graveyard mg42 active");
		
		if (level.graveyardMG42TargettingGuy)
			setGraveyardMG42Target(level.graveyardMG42TargettedGuy);
	}
}

graveyardMG42RandomTarget(stopEvent)
{
	level endon("graveyard mg42 death");
	level endon(stopEvent);
	
	mg42 = getent("graveyard mg42", "targetname");
	while(1)
	{
		wait(2 + randomfloat(4));
		while(1) // wait until the mg42 isn't targetting a player and is active
		{
			ok = true;
			if (level.graveyardMG42TargettingGuy) {
				ok = false;
				level waittill("graveyard mg42 no longer targeting guy");
			}
			if (!level.graveyardMG42Active) {
				ok = false;
				level waittill("graveyard mg42 active");
			}
			if (ok)
				break;
		}

		which = randomint(3);

		target = getent("graveyard mg42 target " + which, "targetname");
		mg42.manual_target = target;
		mg42 settargetentity(target);
		mg42 setmode("manual_ai");
	}
}
setGraveyardMG42Target(guy)
{
	mg42 = getent("graveyard mg42", "targetname");

	level.graveyardMG42TargettingGuy = true;
	level.graveyardMG42TargettedGuy = guy;

	mg42 = getent("graveyard mg42", "targetname");
	//mg42 notify("stopfiring");
	mg42.manual_target = guy;
	mg42 settargetentity(guy);
	//mg42 setmode("auto_ai");
	
	level notify("graveyard mg42 target guy changed");

	thread graveyardMG42GoEasyOnTarget(guy);
	thread graveyardMG42StopTargettingIfGuyDies(guy);
}
unsetGraveyardMG42Target()
{
	level.graveyardMG42TargettingGuy = false;
	level notify("graveyard mg42 no longer targeting guy");

	mg42 = getent("graveyard mg42", "targetname");
	mg42 notify("startfiring"); // in case it stopped while targetting the guy beacuse of graveyardMG42GoEasyOnTarget
}
graveyardMG42StopTargettingIfGuyDies(guy)
{
	level endon("graveyard mg42 no longer targeting guy");
	level endon("graveyard mg42 target guy changed");
	guy waittill("death");

	mg42 = getent("graveyard mg42", "targetname");

	// set its new target immediately, just in case
	target = getent("graveyard mg42 target 1", "targetname");
	mg42.manual_target = target;
	mg42 settargetentity(target);

	unsetGraveyardMG42Target();
}
graveyardMG42GoEasyOnTarget(guy)
{
	level endon("graveyard mg42 no longer targeting guy");
	level endon("graveyard mg42 target guy changed");
	guy endon("death");

	mg42 = getent("graveyard mg42", "targetname");
	while(1)
	{
		guy waittill("pain");
		if (!isdefined(guy.magic_bullet_shield)) {
			while(1) {
				if (isdefined(guy.magic_bullet_shield))
					break;
				wait(.2);
			}
			//continue;
		}
		mg42 notify("stopfiring");
		wait(1.5 + randomfloat(2));
		mg42 notify("startfiring");
	}
}

// these next four functions let us set the graveyard's MG42 non player targets while 
// still being able to temporarily override them and target the player
setGraveyardMG42NonPlayerTarget(guy)
{
	level.graveyardMG42HasNonPlayerTarget = true;
	level.graveyardMG42NonPlayerTarget = guy;
	if (!level.graveyardMG42TargettingPlayer)
		setGraveyardMG42Target(guy);
}
unsetGraveyardMG42NonPlayerTarget()
{
	level.graveyardMG42HasNonPlayerTarget = false;
	if (!level.graveyardMG42TargettingPlayer)
		unsetGraveyardMG42Target();
}
graveyardMG42TargetPlayer()
{
	if (!level.graveyardMG42TargettingPlayer)
	{
		level.graveyardMG42TargettingPlayer = true;
		setGraveyardMG42Target(level.player);
	}
}
graveyardMG42UntargetPlayer()
{
	if (level.graveyardMG42TargettingPlayer)
	{
		level.graveyardMG42TargettingPlayer = false;
		if (level.graveyardMG42HasNonPlayerTarget)
			setGraveyardMG42Target(level.graveyardMG42NonPlayerTarget);
		else
			unsetGraveyardMG42Target();
	}
}

protectPlayerWhileInChurchOrHouse()
{
	level endon("stop protecting player while in church or house");
	
	secondsSinceWarning = 100;
	
	churchMainArea = getent("church main area", "targetname");
	churchExitArea = getent("church exit area", "targetname");
	houseArea = getent("inside house", "targetname");
	while(1)
	{
		level.player.ignoreme = false;
		if (level.player istouching(churchMainArea) || level.player istouching(churchExitArea) || level.player istouching(houseArea))
		{
			if (level.protectWithIgnoring)
				level.player.ignoreme = true;
			level.player.threatbias = 0;
			if (isdefined(level.graveyardMG42Inited))
				graveyardMG42UntargetPlayer();
		}
		else
		{
			if (secondsSinceWarning > 7)
			{
				// warning to get back where it's safe
				if (level.graveyardState == "in church") {
					level.baker thread anim_single_solo (level.baker, "take cover");
					secondsSinceWarning = 0;
				}
				else if (level.graveyardState == "in house") {
					level.foley thread anim_single_solo (level.foley, "take cover");
					secondsSinceWarning = 0;
				}
			}
			level.player.threatbias = 9999999;
			if (isdefined(level.graveyardMG42Inited))
				graveyardMG42TargetPlayer();
		}
		wait(1);
		secondsSinceWarning++;
	}
}

protectBasedOnDeathRate(uniqueID, guy, spawnIDToKill, maxKillInterval, stopEvent)
{
	if (isdefined(stopEvent))
	{
		if (hasEventOccured(stopEvent))	
			return;
		level endon (stopEvent);
	}
	
	guy endon("death");
	
	if (!isdefined(guy.magic_bullet_shield))
		guy thread maps\_utility::magic_bullet_shield();
	
	prevkills = getSpawnCurNumDeaths(spawnIDToKill);
	while(1)
	{
		// wait until either the interval passes or a guy dies
		thread protectBasedOnDeathRateTimer(uniqueID, maxKillInterval);
		spawnDeathEvent(spawnIDToKill, prevkills + 1, "protection kill " + uniqueID);
		waitAnyEvent("protection time ran out " + uniqueID, "protection kill " + uniqueID);
		
		// if no one's died, we stop protecting the guy
		if (getSpawnCurNumDeaths(spawnIDToKill) == prevkills)
		{
			debug ("stopping protection");
			guy notify("stop magic bullet shield");
			// and wait until someone dies
			waitEvent("protection kill " + uniqueID);
			// then start protecting him again and continue
			debug ("starting protection");
			guy thread maps\_utility::magic_bullet_shield(guy.health);
		}
		else
			level notify("protection stop timer " + uniqueID);

		// undo the kill event so we don't think a guy dies immediately next time around...
		undoEvent("protection kill " + uniqueID);
		
		prevkills = getSpawnCurNumDeaths(spawnIDToKill);
	}
}

protectBasedOnDeathRateTimer(uniqueID, time)
{
	level endon("protection stop timer " + uniqueID);
	wait(time);
	level notify("protection time ran out " + uniqueID);
}

moveFoleyWhenPlayerArrivesFromGraveyard(stopEvent)
{
	level endon(stopEvent);
	
	if (!hasEventOccured("player entered house"))
	{
		trigger = getent("near house", "targetname");
		trigger waittill("trigger");
	}
	
	level.foley setGoalNode(getnode("foley house defend", "targetname"));
}

missionFailFoleyDies()
{
	level endon("foley survives graveyard");
	
	level.foley waittill("death");
	
	objective_state(2, "failed");
	wait 1;
	setCvar("ui_deadquote", "@MYDAWNVILLE_FAIL_FOLEY_DIES");
	missionfailed();
}
missionFailBakerDies()
{
	level endon("baker survives graveyard");
	
	level.baker waittill("death");
	
	objective_state(4, "failed");
	wait 1;
	setCvar("ui_deadquote", "@MYDAWNVILLE_FAIL_BAKER_DIES");
	missionfailed();
}

// -------------------------------------
//  MORTAR BATTLE
// -------------------------------------

mortar_battle()
{
	// wait until either the graveyard fight is over, or the player tries to leave the graveyard area
	triggeredEvent("player left graveyard", "player left graveyard");
	waitAnyEvent("player left graveyard", "graveyard finished");
	
	if (isdefined(level.graveyardquick) && level.graveyardquick)
		eventOccur("graveyard finished"); // we didn't do this earlier, to delay getting to this point until the player passes through the trigger
	
	thread mortar_battle_objectives();
	
	// spawn some guys to fight
	if (!isdefined(level.mortarquick) || !level.mortarquick)
	{
		setMaxSpawn("spawn ID mortar defenders", 3);
		thread jitterSpawn(
			"mortar defenders", // targetname of spawners
			"spawn ID mortar defenders", // spawn ID to use for spawned characters
			2, 4, // min-max spawn interval
			"all mortars dead", // stop spawning when this happens
			"mortar defender spawn complete" // throw this event when finished spawning
		);
	}
	
	// make sure the graveyard fight is over before moving foley and baker on
	waitEvent("graveyard finished");
	
	thread fight3Preparation();
	
	autoSave(3);
	
	level.foley thread anim_single_solo (level.foley, "mortars outta business");
	
	level.foley setGoalNode(getnode("foley mortar attack", "targetname"));
	level.baker setGoalNode(getnode("baker mortar attack", "targetname"));
	
	// let 'em roam free
	level.foley.goalradius = 512;
	level.baker.goalradius = 512;
	
	// wait until the mortar teams have been taken out and the defenders have all died
	if (!isdefined(level.mortarquick) || !level.mortarquick)
	{
		spawnFullDeathEvent("spawn ID mortar defenders", "all mortar defenders dead", "mortar defender spawn complete");
		waitAllEvents("all mortars dead", "all mortar defenders dead");
	}
	
	// move foley and baker to the door
	level.foley setGoalNode(getnode("foley door kick", "targetname"));
	level.baker setGoalNode(getnode("baker door wait", "targetname"));
	level.foley.goalradius = 4;
	level.baker.goalradius = 4;
	level.foley allowedstances("stand");
	level.baker allowedstances("stand");
	
	level.foley waittill("goal");
	
	// wait for player to be near door
	neardoortrig = getent("near door", "targetname");
	neardoortrig waittill("trigger");
	
	// kick it open
	door = getent("door", "targetname");
	level.foley thread anim_single_solo (level.foley, "kick door");
	wait(.5);
	door playsound ("wood_door_kick");
	door rotateyaw(-100, .25, .03, 0);
	door waittill("rotatedone");
	door connectpaths();
	door playsound ("door_bounce");
	door rotateyaw(13, 4, 0, 2);
	
	level.foley allowedstances("prone", "crouch", "stand");
	level.baker allowedstances("prone", "crouch", "stand");
	
	// even though we will soon tell foley and baker to follow the player,
	// this ensures foley doesn't just stand there for a moment
	// after kicking open the door.
	level.foley setGoalNode(getnode("foley after door", "targetname"));

	thread fight3();
	
	wait(.1);
	autoSave(4);
}

mortar_battle_objectives()
{
	objnum = 6;
	
	waitEvent("graveyard finished");

	objective_state(objnum-1, "done");

	objective_add(objnum, "active", "", (0, 0, 0));

	if (!isdefined(level.mortarquick) || !level.mortarquick)
	{
		while(1)
		{
			numMortars = level.mortar_team_position.size;

			objective_string(objnum,&"MYDAWNVILLE_OBJ_KILL_MORTARS", numMortars);
			if (numMortars > 0)
				objective_position(objnum, level.mortar_team_position[numMortars-1]);

			objective_current(objnum);

			if (numMortars == 0)
				break;

			if (numMortars == 1) {
				// reduce mortar fall rate
				level.mortarMinInterval = 8;
				level.mortarMaxInterval = 12;
			}

			level waittill ("Mortar Team Dispatched");
		}
	}
	
	objective_state(objnum, "done");
	
	eventOccur("all mortars dead");
	
	// start killing off the mortar defenders so things progress quickly
	thread killOffSlowly(getGuysBySpawnID("spawn ID mortar defenders"), 3, 6);
	
	objective_add(objnum+1, "active", &"MYDAWNVILLE_OBJ_MEET_SQUAD", getent("squad objective location", "targetname").origin);
	objective_current(objnum+1);
}

// -------------------------------------
//  FIGHT 3
// -------------------------------------

fight3()
{
	// scale up the fight a little
	setMaxSpawn("spawn ID fight3 axis", 12);
	
	// make allies non-invincible
	// and add them to the new spawn ID we'll be using for allied units
	allies = getaiarray("allies");
	for (i = 0; i < allies.size; i++)
	{
		if (allies[i] == level.foley)
			continue;
		if (allies[i] == level.baker)
			continue;
		allies[i] notify("stop magic bullet shield");
		switchSpawnID(allies[i], "spawn ID fight3 allies");
	}
	eventOccur("stop ignoring allies");
	
	// get some reinforcements coming
	setMaxSpawn("spawn ID fight3 allies", 4); // will be + 2 (baker and foley)
	thread continuousSpawn(
		"fight3 allied spawn", // spawners targetname
		"spawn ID fight3 allies", // spawn ID for the guys spawned
		"stop fight3 allied spawning", // stop spawning if this event happens
		"fight3 allied spawn finished", // throw this event when finished
		3, 7, // min - max interval between spawns
		-1, // number to spawn (-1 = infinite)
		2 // try to keep at least this many alive at once
	);
	
	// make baker and foley behave like everyone else
	level.foley setgoalentity(level.player);
	level.baker setgoalentity(level.player);
	level.foley.goalradius = 256;
	level.baker.goalradius = 256;
	
	// when we set the max spawn for allies or axis, actually take that seriously and kill off guys to meet it
	thread controlSpawnPopulation("spawn ID fight3 allies");
	thread controlSpawnPopulation("spawn ID fight3 axis");
	
	// occasionally give AIs grenades
	thread giveGrenades("spawn ID fight3 axis", .6, 5);
	//thread giveGrenades("spawn ID fight3 allies", .25, 5);
	
	if (!isdefined(level.fight3quick) || !level.fight3quick)
	{
		// watch to see when the player is finished using the mg42
		thread observePlayerMG42Usage("spawn ID fight3 axis", 15, "spawn ID fight3 allies"); // (requires x deaths during usage)

		thread tellPlayerToUseMG42();

		// wait until the player is finished using the MG42
		waitEvent("player used mg42");
	}
	
	// bring in the tank
	thread tank1();
}

fight3Preparation()
{
	// advance allied forces into the houses
	level.player SetFriendlyChain(getnode("fight 3 friendly chain", "targetname"));
	
	// change where the axis forces are spawning from
	
	// stop old spawns
	eventOccur("stop fight2 spawning");
	
	// set old axis forces to the new spawn ID
	oldaxis = getGuysBySpawnID("spawn ID fight2 axis");
	for (i = 0; i < oldaxis.size; i++)
		switchSpawnID(oldaxis[i], "spawn ID fight3 axis");
	
	setMaxSpawn("spawn ID fight3 axis", 12);
	thread continuousSpawn(
		"fight3 axis spawn", // spawners targetname
		"spawn ID fight3 axis", // spawn ID for the guys spawned
		"stop fight3 axis spawning", // stop spawning if this event happens
		"fight3 axis spawn finished", // throw this event when finished
		2, 4, // min - max interval between spawns
		-1, // number to spawn (-1 = infinite)
		8 // try to keep at least this many alive at once
	);
}

observePlayerMG42Usage(axisSpawnID, requiredDeaths, alliedSpawnID)
{
	// watch how long the player uses the MG42 for.
	// we assume that if the player is within the trigger right around the MG42, that they're using it.
	
	usetime = 0;
	wasInTriggerLastTime = false;
	totalnumdeaths = 0;
	
	trigger = getent("fight3 mg42", "targetname");
	
	prevnumdeaths = getSpawnCurNumDeaths(axisSpawnID);
	
	while(1)
	{
		curnumdeaths = getSpawnCurNumDeaths(axisSpawnID);
		if (level.player istouching(trigger))
		{
			setSpawnAccuracy(alliedSpawnID, 0);

			if (!hasEventOccured("player started using mg42"))
				eventOccur("player started using mg42");
			
			if (wasInTriggerLastTime) {
				// see how many deaths occured while the player was in the trigger over the last second
				totalnumdeaths += (curnumdeaths - prevnumdeaths);
				debug("deaths: " + totalnumdeaths);
				if (totalnumdeaths >= requiredDeaths)
					break;
			}
			wasInTriggerLastTime = true;
		}
		else {
			unsetSpawnAccuracy(alliedSpawnID);
			wasInTriggerLastTime = false;
		}
		prevnumdeaths = curnumdeaths;
		wait(1);
	}
	
	// player has killed enough axis
	unsetSpawnAccuracy(alliedSpawnID);
	eventOccur("player used mg42");
}

tellPlayerToUseMG42()
{
	thread showMG42ObjectiveWhenUsed();
	
	if (hasEventOccured("player started using mg42"))
		return;
	level endon("player started using mg42");

	// first, wait a little while and let the player fight normally
	// while we secretly laugh at them for wasting their time, tee hee
	
	// 20 seconds is actually quite reasonable when you consider the time it takes for the player
	// to get through the door and figure out where they are and what's going on
	wait(20);
	
	if (!hasEventOccured("mg42 objective began"))
		thread mg42Objective();
	
	// have foley tell player to use MG42 until they start using it
	while(1)
	{
		level.foley thread anim_single_solo (level.foley, "martin quick");
		
		wait(10 + randomfloat(10));
	}
}
showMG42ObjectiveWhenUsed()
{
	// if the player starts using the MG42, change the objective to tell them to do what they're already doing
	level endon("mg42 objective began");
	
	waitEvent("player started using mg42");
	
	thread mg42Objective();
}
mg42Objective()
{
	eventOccur("mg42 objective began");

	objective_state(7, "done");
	
	objective_add(8, "active", &"MYDAWNVILLE_OBJ_MG42", getent("upstairs mg42 pos", "targetname").origin);
	objective_current(8);
}

// -------------------------------------
//  TANK 1
// -------------------------------------

tank1()
{
	if (!isdefined(level.tank1quick) || !level.tank1quick)
	{
		level.tank1objshown = false;
		
		thread tank1dotank();
	
		waitEvent("tank 1 death");
		
		if (!level.tank1objshown)
			tank1obj();

		objective_state(9, "done");

		wait(3.5);
	}

	// backdoor attack
	thread backdoor();
}

tank1dotank()
{
	tank = spawnVehicle( "xmodel/vehicle_tank_tiger_camo", "tank 1" , "tiger", (0,0,0), (0,0,0) );
	thread tank1AlliedReaction(tank);
	
	thread eventOnDeath(tank, "tank 1 death");
	
	tank endon("death");
	
	tank thread maps\_tiger::init();
	tank.health = 900; // override what it's set to in init()

	path = getvehiclenode("tank 1 start", "targetname");
	
	tank attachpath(path);
	tank startpath();
	
	thread tank1ShootTrig(tank);
	
	// wait until it's gone far enough
	node = getvehiclenode ("tank near tree","targetname");
	tank setwaitnode (node);
	tank waittill ("reached_wait_node");
	
	// aim at wall
	wait(2.5);
	tank setTurretTargetEnt( getent("tank 1 target", "targetname"), ( 0, 0, 0 ) );
	tank notify("aim began");
}

tank1AlliedReaction(tank)
{
	tank endon("death");
	
	wait(3.5);
	
	// get a random allied ai near the player to warn about the tank
	friend = getRandomAllyNearPlayer();
	if (isdefined(friend)) {
		friend.animname = "generic";
		friend thread anim_single_solo (friend, "enemy tank look out");
	}
	
	wait(2);
	autoSave(5);
	
	wait(2);
	level.foley thread anim_single_solo (level.foley, "get a panzerfaust");

	tank1obj();
}
tank1obj()
{
	objective_state(8, "done");
	
	objective_add(9, "active", &"MYDAWNVILLE_OBJ_TANK1", getent("panzerfaust objective location", "targetname").origin);
	objective_current(9);
	
	panzerfaustObjectiveOn();
	
	level.tank1objshown = true;
}

tank1ShootTrig(tank)
{
	tank endon("death");
	
	triggeredEvent("tank 1 explosion", "tank 1 ok to shoot");
	
	tank waittill("aim began");
	tank waittill("turret_on_target");
	waitEvent("tank 1 ok to shoot");
	
	// shoot until we hit the wall
	while(1)
	{
		tank FireTurret();
		wait(2);
		if (hasEventOccured("tank 1 explosion happened"))
			break;
		wait(.1 + randomfloat(1));
	}
}

// -------------------------------------
//  BACK DOOR
// -------------------------------------

backdoor()
{
	thread forceSpawn(getent("axis backdoor attacker", "targetname"), "spawn ID fight3 backdoor axis", "player entered back room");

	objective_add(10, "active", &"MYDAWNVILLE_OBJ_BACKDOOR", getent("backroom objective location", "targetname").origin);
	objective_current(10);
	
	// take some allied players and send them to the back entrance, switch their spawn ID
	freeallies = getGuysBySpawnID("spawn ID fight3 allies");
	freeallies = maps\_utility::array_randomize(freeallies);
	
	backdoornodes = getNodeArrayFromChain("backdoor room chain");
	
	for (i = 0; i < freeallies.size && i < 2; i++) {
		switchSpawnID(freeallies[i], "spawn ID fight3 backdoor allies");
		freeallies[i] setGoalNode(backdoornodes[randomint(backdoornodes.size)]);
		freeallies[i].goalradius = 32;
		freeallies[i].suppressionwait = 0;
	}
	
	// start spawning some allied ais to go to the back entrance, keep 1-3 there
	setMaxSpawn("spawn ID fight3 backdoor allies", 2);
	setSpawnAccuracy("spawn ID fight3 backdoor allies", 0); // make the player do the work
	thread continuousSpawn(
		"allied backdoor spawn", // spawners targetname
		"spawn ID fight3 backdoor allies", // spawn ID for the guys spawned
		"backdoor objective finished", // stop spawning if this event happens
		"allied backdoor spawn finished", // throw this event when finished
		2, 5, // min - max interval between spawns
		-1, // number to spawn (-1 = inf)
		1 // try to keep at least this many alive at once
	);
	
	//thread noticeAlliedBackdoorSpawners();
	
	// spawn backdoor axis attackers
	setMaxSpawn("spawn ID fight3 backdoor axis", 7);
	thread continuousSpawn(
		"backdoor axis spawn", // targetname of spawners
		"spawn ID fight3 backdoor axis", // spawn ID to use for spawned characters
		"backdoor objective finished", // stop spawning when this happens
		"axis backdoor spawn finished", // throw this event when finished spawning
		2, 4, // min-max spawn interval
		15, // number to spawn
		4 // try to keep at least this many alive at once
	);
	
	// reduce the number of allies and axis in the main fight to compensate
	setMaxSpawn("spawn ID fight3 allies", 2); // used to be 4
	setMaxSpawn("spawn ID fight3 axis", 8); // used to be 12
	
	// occasionally give axis grenades
	thread giveGrenades("spawn ID fight3 backdoor axis", .2, 3);
	
	if (!isdefined(level.backdoorquick) || !level.backdoorquick)
	{
		// tell player to go to the back door
		thread tellPlayerToGoToBackDoor();
	
		// wait for player to arrive at back entrance
		trigger = getent("backdoor area", "targetname");
		trigger waittill("trigger");
		eventOccur("player entered back room");

		autoSave(6);

		// wait until the player kills most of the axis ais (all but 2)
		spawnDeathEvent("spawn ID fight3 backdoor axis", 14, "most backdoor axis dead");
		waitEvent("most backdoor axis dead");
	}
	
	// finished!
	eventOccur("backdoor objective finished");
	
	unsetSpawnAccuracy("spawn ID fight3 backdoor allies"); // return accuracy to normal

	// gogo second tank
	thread tank2();

	wait(5);
	// scale the main fight back up
	if (!hasEventOccured("tank 2 death"))
	{
		setMaxSpawn("spawn ID fight3 allies", 4);
		setMaxSpawn("spawn ID fight3 axis", 12);
	}
}

tellPlayerToGoToBackDoor()
{
	level endon("player entered back room");
	
	while(1)
	{
		friend = getRandomAllyNearPlayer();
		if (isdefined(friend)) {
			friend.animname = "generic";
			friend thread anim_single_solo (friend, "germans backdoor");
		}
		
		wait(8 + randomfloat(5));
	}
}


// (for debugging)
noticeAlliedBackdoorSpawners()
{
	while(1)
	{
		guys = getGuysBySpawnID("spawn ID fight3 backdoor allies");
		for (i = 0; i < guys.size; i++) {
			print3d(guys[i].origin + (0, 0, 50), "ALLIED GUY HERE!", (1,1,1));
		}
		wait(.1);
	}
}

// -------------------------------------
//  TANK 2
// -------------------------------------

tank2()
{
	if (!isdefined(level.tank2quick) || !level.tank2quick)
	{
		level.tank2objshown = false;
		
		thread tank2dotank();

		waitEvent("tank 2 death");
		
		if (!level.tank2objshown)
			tank2obj();
	
		objective_state(11, "done");
	}
	else {
		wait(5);
		eventOccur("tank 2 death");
	}
	
	// wrap things up
	
	// close the back door so the player can't leave during the end timer
	thread closedoor();
	
	// take the allies who are near the backdoor and kill them
	boredallies = getGuysBySpawnID("spawn ID fight3 backdoor allies");
	for (i = 0; i < boredallies.size; i++) {
		boredallies[i] DoDamage(boredallies[i].health + 50, (0,0,0));
	}
	
	// more axis, fewer allies
	setMaxSpawn("spawn ID fight3 allies", 3); // (plus baker and foley)
	setMaxSpawn("spawn ID fight3 axis", 15);

	thread reinforcements();
	
	wait(3);
	
	autoSave(8);
}

tank2dotank()
{
	tank = spawnVehicle( "xmodel/vehicle_tank_tiger_camo", "tank 2" , "tiger", (0,0,0), (0,0,0) );
	thread tank2AlliedReaction(tank);

	thread eventOnDeath(tank, "tank 2 death");
	tank endon("death");
	
	triggeredEvent("tank 2 explosion", "tank 2 ok to shoot");

	tank thread maps\_tiger::init();
	tank.health = 1500; // override what it's set to in init()
	
	tank thread splash_shield();

	path = getvehiclenode("tank 2 start", "targetname");
	
	tank attachpath(path);
	tank startpath();
	
	// wait until it's gone far enough
	node = getvehiclenode ("tank 2 near goal", "targetname");
	tank setwaitnode (node);
	tank waittill ("reached_wait_node");

	// aim at wall
	tank setTurretTargetEnt( getent("tank 2 target 1", "targetname"), ( 0, 0, 0 ) );
	tank waittill("turret_on_target");
	waitEvent("tank 2 ok to shoot");

	// shoot until we hit the wall
	while(1)
	{
		tank FireTurret();
		wait(1.5);
		if (hasEventOccured("tank 2 explosion 1 happened"))
			break;
		wait(.1 + randomfloat(2));
	}

	// wait briefly
	wait(1.2);

	// aim at second wall
	tank setTurretTargetEnt( getent("tank 2 target 2", "targetname"), ( 0, 0, 0 ) );
	tank waittill("turret_on_target");

	// shoot until we hit the wall
	while(1)
	{
		tank FireTurret();
		wait(1.5);
		if (hasEventOccured("tank 2 explosion 2 happened"))
			break;
		wait(.1 + randomfloat(1));
	}
}

tank2AlliedReaction(tank)
{
	tank endon("death");
	
	tanknearbytrig = getent("tank 2 trigger", "targetname");
	tanknearbytrig waittill("trigger");
	
	// get a random allied ai near the player to warn about the tank
	friend = getRandomAllyNearPlayer(getent("backdoor area", "targetname"));
	if (isdefined(friend)) {
		friend.animname = "generic";
		friend thread anim_single_solo (friend, "tank to the south");
	}
	
	wait(5);
	level.foley thread anim_single_solo (level.foley, "get another panzerfaust");

	wait(2);
	autoSave(7);
		
	tank2obj();
}
tank2obj()
{
	objective_state(10, "done");
	
	objective_add(11, "active", &"MYDAWNVILLE_OBJ_TANK2", getent("panzerfaust objective location", "targetname").origin);
	objective_current(11);
	
	panzerfaustObjectiveOn();
	
	level.tank2objshown = true;
}

// taken from dawnville, slightly modified
// solves the tank-killing-itself problem.
splash_shield()
{
	healthbuffer = 10000;
	self.health += healthbuffer;
	while(self.health > 0)
	{
		oldhealth = self.health;
		self waittill ("damage", amount, attacker);
		
		if (attacker != level.player)
			self.health = oldhealth;
		else if (self.health <= healthbuffer)
			break;
	}
	radiusDamage ( self.origin, 2, healthbuffer + 1000, healthbuffer + 900);
}

closedoor()
{
	door = getent("door", "targetname");
	door rotateyaw(87, .5, 0, 0);
	door waittill("rotatedone");
	door disconnectpaths();
}

// -------------------------------------
//  REINFORCEMENTS
// -------------------------------------

reinforcements()
{
	objective_add(12, "active", &"MYDAWNVILLE_OBJ_REINFORCEMENTS", getent("squad objective location", "targetname").origin);
	objective_current(12);
	
	// start timer
	level.reinforcementsTime = 1.5*60;
	level.endTime = 13;
	
	if (isdefined(level.reinforcementsquick) && level.reinforcementsquick)
		level.reinforcementsTime = 30;

	level.hudTimerIndex = 20;

	level.timer = newHudElem();
	level.timer.alignX = "left";
	level.timer.alignY = "middle";
	level.timer.x = 460;
	level.timer.y = 100;
	level.timer.label = &"MYDAWNVILLE_TIMER_TEXT";
	level.timer setTimer(level.reinforcementsTime);
	
	thread endMusic();
	thread protectPlayer();
	thread axisCharge();
	
	reinforcementsArriveBuffer = 3;

	wait(level.reinforcementsTime - reinforcementsArriveBuffer);
	addReinforcements();
	wait(reinforcementsArriveBuffer);
	
	level.timer destroy();
	objective_state(12, "done");
	thread axisRetreat();
	
	wait(level.endTime);
	
	// wait for all axis to die, for a maximum of 5 seconds
	timedEvent(5, "game over timer");
	spawnFullDeathEvent("spawn ID fight3 axis", "all fight3 axis dead", "fight3 axis spawn 2 finished", "game over");
	waitAnyEvent("all fight3 axis dead", "game over timer");
	
	eventOccur("game over");
	
	missionSuccess("credits", false);
}

addReinforcements()
{
	debug("adding reinforcements");
	
	// actually create more allied players
	eventOccur("stop fight3 allied spawning"); // stop earlier continuous spawn so we can start a new one
	setMaxSpawn("spawn ID fight3 allies", 13);
	thread continuousSpawn(
		"fight3 allied spawn", // spawners targetname
		"spawn ID fight3 allies", // spawn ID for the guys spawned
		"stop fight3 allied reinforcements", // stop spawning if this event happens
		"fight3 allied reinforcements spawn finished", // throw this event when finished
		1, 3, // min - max interval between spawns
		-1, // number to spawn (-1 = infinite)
		9 // try to keep at least this many alive at once
	);
	
	// set allied spawners to go up to wall instead of into houses
	alliedspawners = getentarray("fight3 allied spawn", "targetname");
	newtargets = getNodeArrayFromChain("allies push to");
	newtargets = maps\_utility::array_randomize(newtargets);
	for (i = 0; i < alliedspawners.size; i++)
	{
		alliedspawners[i].target = newtargets[i].targetname;
	}

	// at the same time, increase the flow of axis players so there will be some there to retreat
	eventOccur("stop fight3 axis spawning");
	thread continuousSpawn(
		"fight3 axis spawn", // spawners targetname
		"spawn ID fight3 axis", // spawn ID for the guys spawned
		"axis retreat", // stop spawning if this event happens
		"fight3 axis spawn 2 finished", // throw this event when finished
		1, 3, // min - max interval between spawns
		-1, // number to spawn (-1 = infinite)
		12 // try to keep at least this many alive at once
	);
}

endMusic()
{
	// music makes everything cooler =)
	
	wait(level.reinforcementsTime + level.endTime - 35);
	musicPlay("music_victory");
}

protectPlayer()
{
	if (level.reinforcementsTime > 45)
		wait(level.reinforcementsTime - 45);
	// At this point, I'd rather the player have a good time than die.
	while(1)
	{
		level.player waittill("damage");
		
		if (getHealthAmnt() > .4)
			continue;
		
		level.player.ignoreme = true;
		
		while(getHealthAmnt() < .5)
			wait(3);
		
		level.player.ignoreme = false;
	}
}
getHealthAmnt()
{
	return (float)level.player.health / (float)level.player.maxhealth;
}

axisCharge()
{
	if (!isdefined(level.reinforcementsquick) || !level.reinforcementsquick)
		wait(20);
	
	debug("AXIS CHARGE!");
	
	// make new axis ais stop hiding behind the wall and charge the houses
	axisspawners = getentarray("fight3 axis spawn", "targetname");
	for (i = 0; i < axisspawners.size; i++)
	{
		curtarget = getnode(axisspawners[i].target, "targetname");
		axisspawners[i].target = curtarget.target;
	}
	
	// ensure that the existing axis get killed off so new ones can come and start charging
	thread killOffSlowly(getGuysBySpawnID("spawn ID fight3 axis"), 1, 2);
	
	// make them less likely to die
	setSpawnAccuracy("spawn ID fight3 allies", .15);
	level.foley.accuracy = .15;
	level.baker.accuracy = .15;
	
	level endon("axis retreat");
	
	while(1)
	{
		axis = getGuysBySpawnID("spawn ID fight3 axis");
		for (i = 0; i < axis.size; i++)
		{
			axis[i].suppressionwait = 0;
		}
		wait(3);
	}
}

axisRetreat()
{
	debug("AXIS RETREAT!");
	
	// (stop spawning axis ais)
	eventOccur("axis retreat");
	
	// send existing axis running away
	axis = getGuysBySpawnID("spawn ID fight3 axis");
	retreatnodes = getNodeArrayFromChain("axis end retreat to");
	for (i = 0; i < axis.size; i++) {
		axis[i] thread doRetreat(retreatnodes[randomint(retreatnodes.size)]);
		axis[i].accuracy = 0;
	}
	thread killOnTouch(getent("axis retreat death", "targetname"));
	
	wait(4);
	
	level.foley thread anim_single_solo (level.foley, "pulling back");
	
	wait(1);

	// send allies running after the axis, one after another
	newtargets = getNodeArrayFromChain("allies push to");
	allies = getGuysBySpawnID("spawn ID fight3 allies");
	allies[allies.size] = level.foley;
	allies[allies.size] = level.baker;
	for (i = 0; i < allies.size; i++)
	{
		// (skip dead allies)
		for (; i < allies.size; i++) {
			if (isalive(allies[i]))
				break;
		}
		if (i >= allies.size)
			break;
		
		goalnode = newtargets[randomint(newtargets.size)];
		allies[i] setGoalNode(goalnode);
		allies[i].goalradius = goalnode.radius;
		
		wait(1.5 + randomfloat(1));
	}

	// make them more likely to die
	setSpawnAccuracy("spawn ID fight3 allies", .9);
	level.foley.accuracy = .9;
	level.baker.accuracy = .9;
}

killOnTouch(trig)
{
	while(1)
	{
		trig waittill("trigger", guy);
		if (isalive(guy))
			guy DoDamage(guy.health + 50, (randomint(5000)-2500,randomint(5000)-2500,randomint(5000)-2500));
	}
}

// *****************************************************************
// Spawn functions

initSpawns()
{
	level.spawncounts = [];
	level.spawndeaths = [];
	level.maxspawns = [];
	level.spawntargets = [];
	level.spawnignoreme = [];
	level.spawnaccuracy = [];
}

setMaxSpawn(spawnid, max) // sets the maximum number of ai's that can spawn for one spawnid
{
	level.maxspawns[spawnid] = max;
}
setSpawnIgnoreMe(spawnid, val)
{
	level.spawnignoreme[spawnid] = val;
	guys = getGuysBySpawnID(spawnid);
	for (i = 0; i < guys.size; i++)
		guys[i].ignoreme = val;
}
setSpawnAccuracy(spawnid, val)
{
	if (!isdefined(level.spawnaccuracy[spawnid]))
		level.spawnaccuracy[spawnid] = -1;

	if (val < 0) {
		unsetSpawnAccuracy(spawnid);
		return;
	}
	oldval = level.spawnaccuracy[spawnid];
	level.spawnaccuracy[spawnid] = val;
	guys = getGuysBySpawnID(spawnid);
	for (i = 0; i < guys.size; i++) {
		if (oldval < 0)
			guys[i].originalaccuracy = guys[i].accuracy;
		guys[i].accuracy = val;
	}
}
unsetSpawnAccuracy(spawnid)
{
	if (!isdefined(level.spawnaccuracy[spawnid]))
		level.spawnaccuracy[spawnid] = -1;
	
	if (level.spawnaccuracy[spawnid] < 0)
		return;
	level.spawnaccuracy[spawnid] = -1;
	guys = getGuysBySpawnID(spawnid);
	for (i = 0; i < guys.size; i++)
		guys[i].accuracy = guys[i].originalaccuracy;
}

// continuously spawns randomly from spawners with targetname = spawnTargetName, giving the spawned guys the spawn id spawnID,
// stopping when stopEvent happens, and throwing finishedEvent when done.
// waits minInterval to maxInterval between each spawn, spawns maxSpawns (use -1 for infinite) guys.
// if there are less than maintainCount guys with spawn id spawnID, it will try to spawn faster.
continuousSpawn(spawnTargetName, spawnID, stopEvent, finishedEvent, minInterval, maxInterval, maxSpawns, maintainCount)
{
	spawners = getentarray(spawnTargetName, "targetname");
	
	if (isdefined(stopEvent))
	{
		eventOnOtherEvent(stopEvent, finishedEvent);
		if (hasEventOccured(stopEvent))	
			return;
		level endon (stopEvent);
	}
	
	soonaspossible = false;
	
	infinite = false;
	if (maxSpawns < 0 || !isdefined(maxSpawns)) {
		infinite = true;
		numSpawns = 10;
	}
	else {
		numSpawns = maxSpawns;
	}
	
	for (i = 0; i < numSpawns; i++)
	{
		// "if the desired number of spawned units is less than maintainCount"
		if (isdefined(maintainCount) && maintainCount > 0 && (!isdefined(level.spawncounts[spawnID]) || level.spawncounts[spawnID] < maintainCount))
			soonaspossible = true;
		
		if (soonaspossible)
			wait(.1); // don't wait if we have to keep respawning to maintain the desired number of alive units
		else
		{
			waittime = minInterval + randomfloat(maxInterval - minInterval);
			if (isdefined(maintainCount) && maintainCount > 0)
			{
				starttime = gettime();
				waittime = waittime * 1000;
				// check periodically if we need to spawn more units to maintain maintainCount
				while(1)
				{
					timeleft = (starttime + waittime) - gettime();
					if (timeleft <= 500)
						wait (.5);
					else {
						wait (timeleft / 1000);
						break;
					}

					if (!isdefined(level.spawncounts[spawnID]) || level.spawncounts[spawnID] < maintainCount) {
						soonaspossible = true;
						break;
					}
				}
			}
			else
				wait(waittime);
		}
		
		// finished waiting, now try to spawn
		
		if (infinite)
			spawntext = "";
		else
			spawntext = " " + (i+1) + " out of " + numSpawns;
		
		while (1)
		{
			spawner = spawners[randomint(spawners.size)];
			result = spawner spawnGuy(spawnID);
			if (result) {
				//debug("continuousSpawn: Spawned" + spawntext + "; spawncount = " + level.spawncounts[spawnID]);
				break;
			}
			//debug("continuousSpawn: Spawn" + spawntext + " failed (" + level.spawnStatus + "); spawncount = " + level.spawncounts[spawnID]);
			
			if (soonaspossible)
				wait(.1);
			else
				wait(1);
		}
		
		soonaspossible = false;
		
		if (infinite)
			i = 0; // don't stop
	}
	
	//debug("Spawned maxSpawns, stopping");
	if (isdefined(finishedEvent))
		eventOccur(finishedEvent);
}

// tries to spawn numSpawns randomly from spawners with targetname = spawnTargetName, giving the spawned guys the
// spawn id spawnID, stopping when stopEvent happens.
// tries to spawn numSpawns guys as fast as possible, only hesitating if a spawn fails
quickSpawn(spawnTargetName, spawnID, stopEvent, numSpawns)
{
	if (isdefined(stopEvent))
	{
		if (hasEventOccured(stopEvent))
			return;
		level endon (stopEvent);
	}

	spawners = getentarray(spawnTargetName, "targetname");

	for (i = 0; i < numSpawns; i++)
	{
		while (1)
		{
			spawner = spawners[randomint(spawners.size)];
			result = spawner spawnGuy(spawnID);
			if (result) {
				//debug("quickSpawn: Spawned " + (i+1) + " out of " + numSpawns);
				break;
			}
			//debug("quickSpawn: Spawn " + (i+1) + " out of " + numSpawns + " failed (" + level.spawnStatus + ")");
			wait(1);
		}
	}
}

// tries to spawn one guy from each spawner with targetname = spawnTargetName, in random order,
// giving the spawned guys the spawn id spawnID, stopping when stopEvent happens, and throwing finishedEvent when done.
// waits minInterval to maxInterval between each spawn
jitterSpawn(spawnTargetName, spawnID, minInterval, maxInterval, stopEvent, finishedEvent)
{
	if (isdefined(stopEvent))
	{
		eventOnOtherEvent(stopEvent, finishedEvent);
		if (hasEventOccured(stopEvent))
			return;
		level endon (stopEvent);
	}
	
	spawners = getentarray(spawnTargetName, "targetname");
	spawners = maps\_utility::array_randomize(spawners);
	
	for (i = 0; i < spawners.size; i++)
	{
		while (1)
		{
			result = spawners[i] spawnGuy(spawnID);
			if (result) {
				//debug("jitterSpawn: Spawned " + (i+1) + " out of " + spawners.size);
				break;
			}
			//debug("jitterSpawn: Spawn " + (i+1) + " out of " + spawners.size + " failed (" + level.spawnStatus + ")");
			
			// spawn failed, try a different one next
			otherindex = randomint(spawners.size - i) + i;
			temp = spawners[i];
			spawners[i] = spawners[otherindex];
			spawners[otherindex] = temp;
			
			wait(.2);
		}
		wait(minInterval + randomfloat(maxInterval - minInterval));
	}

	if (isdefined(finishedEvent))
		eventOccur(finishedEvent);
}

forceSpawn(spawner, spawnID, stopEvent, finishedEvent) // keeps trying to spawn until spawn succeeds, stops trying if stopEvent happens, throws finishedEvent when done
{
	if (isdefined(stopEvent)) {
		eventOnOtherEvent(stopEvent, finishedEvent);
		if (hasEventOccured(stopEvent))
			return;
		level endon (stopEvent);
	}

	while (1)
	{
		result = spawner spawnGuy(spawnID);
		if (result) {
			//debug("forceSpawn: Spawned");
			break;
		}
		//debug("forceSpawn: Spawn failed (" + level.spawnStatus + ")");
		wait(1);
	}
	
	if (isdefined(finishedEvent))
		eventOccur(finishedEvent);
}

spawnGuy(spawnID) // returns true iff spawn succeeds; if returns false, check level.spawnStatus
{
	level.spawnStatus = "succeeded";
	
	if(!isdefined(level.spawncounts[spawnID])) {
		level.spawncounts[spawnID] = 0;
		level.spawndeaths[spawnID] = 0;
	}
	
	// don't go above the max spawns for this spawnid
	if (isdefined(level.maxspawns[spawnID]) && level.spawncounts[spawnID] >= level.maxspawns[spawnID]) {
		level.spawnStatus = "'" + spawnID + "' spawncount is at max";
		return false;
	}
	
	self.count = 1;
	spawn = self dospawn();
	if (maps\_utility::spawn_failed(spawn)) {
		level.spawnStatus = "maps\_utility::spawn_failed returned true";
		return false;
	}

	// spawn has succeeded
	level.spawncounts[spawnID]++;
	
	spawn.spawnID = spawnID;
	thread trackSpawnedGuy(spawn);
	
	if (!isdefined(level.spawnignoreme[spawnID]))
		level.spawnignoreme[spawnID] = false;
	if (!isdefined(level.spawnaccuracy[spawnID]))
		level.spawnaccuracy[spawnID] = -1;
		
	spawn.ignoreme = level.spawnignoreme[spawnID];
	spawn.originalaccuracy = spawn.accuracy;
	if (level.spawnaccuracy[spawnID] >= 0) {
		spawn.accuracy = level.spawnaccuracy[spawnID];
	}
	
	//spawn thread attack();
	
	level.lastSpawn = spawn;
	if (isdefined(spawnID))
		level notify("spawned with spawn ID: " + spawnID);
	
	return true;
}
/*attack()
{
	if (!isdefined (self.target))
	{
		debug("Spawner at origin " + self.origin + " had no target");
		return;
	}
	
	node = getnode (self.target,"targetname");
	self setgoalnode (node);
	self.goalradius = 4;
	
	self endon("death");
	self waittill ("goal");
	if (isdefined (node.script_radius))
		self.goalradius = node.script_radius;
	else
		self.goalradius = 512;
}*/

switchSpawnID(guy, newSpawnID)
{
	if(!isdefined(level.spawncounts[newSpawnID])) {
		level.spawncounts[newSpawnID] = 0;
		level.spawndeaths[newSpawnID] = 0;
	}
	if (!isdefined(level.spawnignoreme[newSpawnID]))
		level.spawnignoreme[newSpawnID] = false;
	if (!isdefined(level.spawnaccuracy[newSpawnID]))
		level.spawnaccuracy[newSpawnID] = -1;

	oldspawnid = guy.spawnID;

	if (isdefined(guy.spawnID))
		level.spawncounts[guy.spawnID]--;
	guy.spawnID = newSpawnID;
	level.spawncounts[guy.spawnID]++;
	
	if (!isdefined(oldspawnid))
		thread trackSpawnedGuy(guy);
	
	guy.ignoreme = level.spawnignoreme[guy.spawnID];
	// switch to new spawnid's accuracy if it has one
	if (level.spawnaccuracy[guy.spawnID] >= 0) {
		if (isdefined(oldspawnid) && isdefined(level.spawnaccuracy[oldspawnid]) && level.spawnaccuracy[oldspawnid] < 0)
			guy.originalaccuracy = guy.accuracy; // only save original accuracy if guy's *current* spawnid doesn't have its accuracy set
		guy.accuracy = level.spawnaccuracy[guy.spawnID];
	}
	// if new spawnid doesn't have an accuracy set, but old one does, revert to spawn's original accuracy
	else if (isdefined(oldspawnid) && isdefined(level.spawnaccuracy[oldspawnid]) && level.spawnaccuracy[oldspawnid] >= 0) {
		guy.accuracy = guy.originalaccuracy;
	}
}

trackSpawnedGuy(guy)
{
	guy waittill("death");
	level.spawncounts[guy.spawnID]--;
	level.spawndeaths[guy.spawnID]++;
	level notify("__spawndeath" + guy.spawnID);
}

// throws the given event when the number of guys with a given spawn ID that have died reaches a certain number
spawnDeathEvent(spawnID, numDeathsRequired, event)
{
	thread spawnDeathEventThread(spawnID, numDeathsRequired, event);
}
spawnDeathEventThread(spawnID, numDeathsRequired, event) // used by spawnDeathEvent
{
	if (!isdefined(level.spawndeaths[spawnID]))
		level.spawndeaths[spawnID] = 0;
	
	while(1)
	{
		if (level.spawndeaths[spawnID] >= numDeathsRequired)
			break;
		level waittill("__spawndeath" + spawnID);
	}
	
	eventOccur(event);
}

getSpawnCurNumDeaths(spawnID)
{
	if (!isdefined(level.spawndeaths[spawnID]))
		level.spawndeaths[spawnID] = 0;
	return level.spawndeaths[spawnID];
}

// throws the given event when all of the guys with a given spawn spawn ID are dead
// if spawnFinishedEvent is provided, the event won't occur until spawnFinishedEvent has happened;
// recommended to use the event that occurs when spawning is complete for this.
// gives up if stopEvent happens.
spawnFullDeathEvent(spawnID, event, spawnFinishedEvent, stopEvent)
{
	thread spawnFullDeathEventThread(spawnID, event, spawnFinishedEvent, stopEvent);
}
spawnFullDeathEventThread(spawnID, event, spawnFinishedEvent, stopEvent) // used by spawnFullDeathEvent
{
	if (isdefined(stopEvent)) {
		if (hasEventOccured(stopEvent))
			return;
		level endon (stopEvent);
	}

	if (isdefined(spawnFinishedEvent))
		waitEvent(spawnFinishedEvent);
		
	if (!isdefined(level.spawncounts[spawnID]))
		level.spawncounts[spawnID] = 0;
	
	while(1)
	{
		if (level.spawncounts[spawnID] == 0)
			break;
		level waittill("__spawndeath" + spawnID);
	}

	eventOccur(event);
}

getGuysBySpawnID(spawnID)
{
	allguys = getaiarray();
	guys = [];
	for (i = 0; i < allguys.size; i++) {
		if (isdefined(allguys[i].spawnID) && allguys[i].spawnID == spawnID)
			guys[guys.size] = allguys[i];
	}
	return guys;
}

// slowly kills off members of the spawn id when there are more than max
// stops if stopEvent happens
controlSpawnPopulation(spawnID, stopEvent)
{
	if (isdefined(stopEvent)) {
		if (hasEventOccured(stopEvent))
			return;
		level endon (stopEvent);
	}

	while(1)
	{
		guys = getGuysBySpawnID(spawnID);
		if (guys.size > level.maxspawns[spawnID])
		{
			// find the guy farthest from the player
			best = -1;
			bestdist = 0;
			for (i = 0; i < guys.size; i++)
			{
				thisdist = distance(guys[i].origin, level.player.origin);
				if (best == -1 || thisdist > bestdist) {
					best = i;
					bestdist = thisdist;
				}
			}
			if (best >= 0)
				guys[best] DoDamage(guys[best].health + 50, (randomint(5000)-2500,randomint(5000)-2500,randomint(5000)-2500));
		}
		wait(1.5 + randomfloat(2.5));
	}
}

// gives grenades to ais as they spawn
giveGrenades(spawnID, probability, numgrenades)
{
	while(1)
	{
		level waittill("spawned with spawn ID: " + spawnID);
		guy = level.lastSpawn;
		if (randomfloat(1) < probability)
			guy.grenadeammo = numgrenades;
	}
}

// *****************************************************************
// Tank stuff

initTankExplosions()
{
	level.tankShellShocked = false;
	
	monclip = getent("tank 2 explosion 1 monclip", "targetname");
	monclip notsolid();
	monclip connectpaths();
	monclip = getent("tank 2 explosion 2 monclip", "targetname");
	monclip notsolid();
	monclip connectpaths();
	
	thread tank1ExplodeTrig();
	thread tank2Explode1Trig();
	thread tank2Explode2Trig();
}

tank1ExplodeTrig()
{
	triggeredEvent("tank 1 damager", "tank 1 explosion go");
	waitEvent("tank 1 explosion go");
	maps\_utility::exploder(101);
	
	if (!level.tankShellShocked && level.player istouching(getent("near tank 1 explosion", "targetname"))) {
		level.tankShellShocked = true;
		maps\_shellshock::main(5.5);
	}
	
	eventOccur("tank 1 explosion happened");
}
tank2Explode1Trig()
{
	triggeredEvent("tank 2 damager 1", "tank 2 explosion 1 go");
	waitEvent("tank 2 explosion 1 go");
	maps\_utility::exploder(102);
	
	// don't let ai's take cover behind destroyed explosion
	thread makeSolid(getent("tank 2 explosion 1 monclip", "targetname"));
	
	if (!level.tankShellShocked && level.player istouching(getent("near tank 2 explosion 1", "targetname"))) {
		level.tankShellShocked = true;
		maps\_shellshock::main(3.5);
	}
	
	eventOccur("tank 2 explosion 1 happened");
}
tank2Explode2Trig()
{
	triggeredEvent("tank 2 damager 2", "tank 2 explosion 2 go");
	waitEvent("tank 2 explosion 2 go");
	maps\_utility::exploder(103);
	
	// don't let ai's take cover behind destroyed explosion
	thread makeSolid(getent("tank 2 explosion 2 monclip", "targetname"));
	
	if (!level.tankShellShocked && level.player istouching(getent("near tank 2 explosion 2", "targetname"))) {
		level.tankShellShocked = true;
		maps\_shellshock::main(3.5);
	}
	
	eventOccur("tank 2 explosion 2 happened");
}

makeSolid(monclip) // waits until no AI are touching the monclip and then makes it solid
{
	while(1)
	{
		ok = true;
		ais = getaiarray();
		for (i = 0; i < ais.size; i++) {
			if (ais[i] istouching(monclip)) {
				ok = false;
				break;
			}
		}
		if (ok)
			break;
		wait(2);
	}
	monclip solid();
	monclip disconnectpaths();
}

// these functions prevent us from turning the panzerfaust objective on when the player doesn't need to pick one up
initPanzerfaust()
{
	thread panzerfaustWatch();
}
panzerfaustWatch()
{
	level.panzerfaustObjectiveOn = false;
	while(1)
	{
		level.pickedUpPanzerfaust = false;
		level waittill ("picked up infinite panzerfaust");
		level.pickedUpPanzerfaust = true;
		
		if (level.panzerfaustObjectiveOn)
			level.panzerfaustObjectiveOn = false;
		else
			level waittill("infinite panzerfaust reset");
	}
}
panzerfaustObjectiveOn()
{
	if (!level.pickedUpPanzerfaust)
	{
		panzerfaust = getent("infinite panzerfaust", "targetname");
		panzerfaust notify ("infinite panzerfaust objective on");
		level.panzerfaustObjectiveOn = true;
	}
	level notify("infinite panzerfaust reset");
}


// *****************************************************************
// Mortar stuff

initMortars()
{
	level._effect["mortar"] = loadfx ("fx/impacts/newimps/dirthit_mortar2daymarked.efx");
	
	level.mortarInterval = 20000; // 20 seconds between usages of the same mortar spot
	mortars = getentarray ("mortar spot","targetname");
	for (i = 0; i < mortars.size; i++)
	{
		mortars[i].usedTime = 0;
	}
}

getMortarNearPlayer(mindist, maxdist)
{
	mortars = getentarray ("mortar spot","targetname");
	currentTime = gettime();
	
	availableMortars = [];
	
	closestIndex = -1;
	
	for (i = 0; i < mortars.size; i++)
	{
		if (mortars[i].usedTime > currentTime)
			continue;

		newdist = distance (level.player.origin, mortars[i].origin);
		if (newdist > mindist && newdist < maxdist)
		{
			availableMortars[availableMortars.size] = mortars[i];
		}
		if (newdist > mindist && (closestIndex == -1 || newdist < closestDist))
		{
			closestIndex = i;
			closestDist = newdist;
		}
	}
	
	if (availableMortars.size == 0) {
		if (closestIndex == -1)
			return undefined;
		return mortars[closestIndex];
	}

	return availableMortars[randomint(availableMortars.size)];
}

doMortarVeryNearPlayer()
{
	doMortarAtPos(level.player.origin + ((randomfloat(1)*2-1) * 100, (randomfloat(1)*2-1) * 100, 0));
}

doMortarNearPlayer()
{
	mortar = getMortarNearPlayer(250, 600);
	if (isdefined(mortar))
		doMortar(mortar);
}

doAnyMortar()
{
	mortar = getMortarNearPlayer(300, 1000);
	if (isdefined(mortar))
		doMortar(mortar);
}

doMortarEffects(pos, damageModifier)
{
	playfx (level._effect["mortar"], pos + (0,0,-76));
	
	myRadiusDamage(pos, 50, 200, 150 * damageModifier);
	
	earthquake(0.3, 3, pos, 850); // scale duration source radius
}
doMortar(mortar)
{
	mortar.usedTime = getTime() + 99999999; // so we don't try to use it again in a different thread
	
	mortar playsound ("mortar_incoming", "sounddone");
	mortar waittill ("sounddone");
	mortar playsound ("mortar_explosion");
	
	doMortarEffects(mortar.origin, 1);

	mortar.usedTime = gettime() + level.mortarInterval;
}

deleteTempOrigin(tempmortar)
{
	tempmortar waittill("sounddone"); // not sure if this is important, but it probably won't hurt
	tempmortar delete();
}
doMortarAtPos(pos)
{
	tempmortar = spawn ("script_origin",pos);
	
	doMortar(tempmortar);
	
	thread deleteTempOrigin(tempmortar);
}

doMortarRelativeToPlayer(dir, dist, damageModifier)
{
	// play sound a distance in front of player in the general direction we want
	dir = vectornormalize(dir);
	fallpos = level.player.origin + maps\_utility::vectorScale(dir, 1024);
	
	fallposorigin = spawn("script_origin", fallpos);
	fallposorigin playsound ("mortar_incoming", "sounddone");
	fallposorigin waittill ("sounddone");
	fallposorigin delete();
	
	// now that it's time to actually let the mortar hit the ground, 
	// have it hit exactly where we want relative to the player
	hitpos = level.player.origin + maps\_utility::vectorScale(dir, dist);
	
	hitposorigin = spawn("script_origin", hitpos);
	hitposorigin playsound ("mortar_explosion");
	thread deleteTempOrigin(hitposorigin);

	doMortarEffects(hitpos, damageModifier);
}

myRadiusDamage(pos, innerrad, outerrad, damage)
{
	players = getaiarray ();
	players[players.size] = level.player;
	
	for (i = 0; i < players.size; i++)
	{
		dist = distance (pos, players[i].origin);
		if (dist < outerrad){
			if (dist < innerrad)
				amount = 1;
			else
				amount = 1 - (dist - innerrad)/(outerrad - innerrad);
			
			players[i] DoDamage(damage*amount, pos);
		}
	}
}

// *****************************************************************
// Animation stuff

initCharacters()
{
	level.baker = getent("baker", "script_noteworthy");
	level.foley = getent("foley", "targetname");

	character\foley::precache();

	level.foley.animname = "foley";
	level.foley character\_utility::new();
	level.foley character\foley::main();
	level.foley.followMin = 4;
	level.foley.followMax = 8;
	level.foley thread animscripts\shared::SetInCombat(60.0);
	
	level.baker.animname = "baker";
	
	// protect allies
	allies = getaiarray("allies");
	for (i = 0; i < allies.size; i++) {
		allies[i] thread maps\_utility::magic_bullet_shield();
	}
	
	// load animation stuff
	
	level.scrsound["baker"]["here they come"]	       = "mydawnville_baker_heretheycome";
	level.scrsound["foley"]["keep heads down"]       	= "mydawnville_foley_keepheadsdown";

	level.scrsound["foley"]["push them back"]	       = "mydawnville_foley_pushemout";

	level.scrsound["foley"]["mortars"]	               = "mydawnville_foley_mortars";
	level.scrsound["baker"]["mortars incoming"]	       = "mydawnville_baker_mortarsincoming";

	level.scrsound["foley"]["to church with me"]	   = "mydawnville_foley_tochurchwithme";
	level.scr_face["foley"]["to church with me"]	   = %dawnville_facial_foley_069_martinbacktochurch;
	//level.scr_anim["foley"]["to church with me"]	   = %dawn_moody_run_and_wave;
	level.scrsound["foley"]["to church with me short"] = "mydawnville_foley_tochurchwithmeshort";
	
	level.scrsound["baker"]["incoming take cover"]	   = "mydawnville_baker_incomingtakecover";

	level.scrsound["foley"]["the church go go go"]	   = "mydawnville_foley_intothechurch";

	level.scrsound["foley"]["graveyard speech"]        = "mydawnville_foley_graveyardspeech";
	level.scrsound["foley"]["keep shooting"]	       = "mydawnville_foley_keepshooting";
	level.scrsound["baker"]["martin you're next"]	   = "mydawnville_baker_martinyourenext";
	level.scrsound["baker"]["go go go"]	               = "mydawnville_baker_gogogo";
	level.scrsound["foley"]["good job son"]            = "mydawnville_foley_goodjobson";
	level.scrsound["baker"]["charge"]                  = "mydawnville_baker_charge";
	level.scrsound["baker"]["take cover"]              = "mydawnville_baker_takecover";
	level.scrsound["foley"]["take cover"]              = "mydawnville_foley_takecover";
	level.scrsound["foley"]["covering fire"]           = "mydawnville_foley_coveringfire";
	level.scrsound["foley"]["okay move move"]          = "mydawnville_foley_okmovemove";

	level.scrsound["foley"]["mortars outta business"]  = "mydawnville_foley_mortarsouttabusiness";

	level.scr_anim["foley"]["kick door"]	           = %chateau_kickdoor1;

	level.scrsound["foley"]["martin quick"]            = "mydawnville_foley_martinquick";
	level.scrsound["generic"]["enemy tank look out"]   = "mydawnville_generic_enemytanklookout";
	level.scrsound["foley"]["get a panzerfaust"]       = "mydawnville_foley_getapanzerfaust";
	level.scrsound["generic"]["germans backdoor"]      = "mydawnville_generic_germansbackdoor";
	level.scrsound["generic"]["tank to the south"]     = "mydawnville_generic_tanktothesouth";
	level.scrsound["foley"]["get another panzerfaust"] = "mydawnville_foley_getanotherpanzerfaust";

	level.scrsound["foley"]["pulling back"]            = "mydawnville_foley_pullingback";
}

anim_single_solo (guy, anime, tag, node, tag_entity)
{
	newguy[0] = guy;
	maps\_anim::anim_single (newguy, anime, tag, node, tag_entity);
}
anim_loop_solo (guy, anime, tag, ender, node, tag_entity)
{
	newguy[0] = guy;
	maps\_anim::anim_loop ( newguy, anime, tag, ender, node, tag_entity );
}

lookAtUponArrival(person, lookatperson)
{
	person waittill("goal");
	person lookat(lookatperson);
}

lookat (ent, timer)
{
	if (!isdefined (timer))
		timer = 10000;

	self animscripts\shared::lookatentity(ent, timer, "alert");
}

initFX()
{
	level._effect["stone"]	= loadfx ("fx/cannon/stone.efx");
	//level._effect["wood"]	= loadfx ("fx/cannon/wood.efx");
	level._effect["dust"]	= loadfx ("fx/cannon/dust.efx");
	//level._effect["glass"]	= loadfx ("fx/cannon/glass.efx");
	//level._effect["dirt"]	= loadfx ("fx/impacts/stukastrafe_dirt.efx");

 	level._effect["smoke1"]	= loadfx ("fx/smoke/dawnville_smoke3.efx");
    level._effect["smoke2"]	= loadfx ("fx/smoke/dawnville_smoke.efx");
    level._effect["smoke3"]	= loadfx ("fx/smoke/dawnville_smoke2.efx");
    level._effect["smoke4"]	= loadfx ("fx/smoke/dawnville_smoke1.efx");
    level._effect["small_windy1"]	= loadfx ("fx/smoke/dawn_small_c1.efx");

	spawnWorldFX();
}

spawnWorldFX()
{
	// taken directly from dawnville
    maps\_fx::loopfx("smoke1", (1776, -16335, 13), 0.3);
    maps\_fx::loopfx("smoke1", (87, -15543, -12), 0.3);
    maps\_fx::loopfx("smoke1", (2634, -14543, 86), 0.3);
    maps\_fx::loopfx("small_windy1", (-13, -17003, 224), 0.3);
    // maps\_fx::loopfx("smoke1", (110, -15829, 164), 0.3);
    maps\_fx::loopfx("smoke1", (970, -14719, -33), 0.3);
    maps\_fx::loopfx("smoke2", (2079, -17550, 90), 0.3);
    maps\_fx::loopfx("smoke4", (-1198, -17377, -70), 0.3);
    maps\_fx::loopfx("smoke3", (-707, -18404, -99), 0.3);
    maps\_fx::loopfx("smoke3", (176, -16747, 195), 0.2);
    maps\_fx::loopfx("smoke3", (-1315, -17211, 134), 0.2);
}

// *****************************************************************
// Helpful functions

timedEventTimer(duration, event) // used by timedEvent
{
	wait(duration);
	debug("Timer '" + event + "' finished");
	eventOccur(event);
}
timedEvent(duration, event) // creates an event when the duration passes
{
	thread timedEventTimer(duration, event);
}
timedEventOnceTimer(duration, event) // used by timedEventOnce
{
	wait(duration);
	debug("Timer '" + event + "' finished");
	level notify(event);
}
timedEventOnce(duration, event)
{
	thread timedEventOnceTimer(duration, event);
}

triggeredEventWaiter(trigger, event, stopEvent) // used by triggeredEvent
{
	if (isdefined(stopEvent))
		level endon(stopEvent);
	trigger waittill("trigger");
	eventOccur(event);
}
triggeredEvent(triggerTargetname, event, stopEvent)
{
	trigger = getent(triggerTargetname, "targetname");
	thread triggeredEventWaiter(trigger, event, stopEvent);
}

debug(text)
{
	//iprintln("Debug: ", text);
}

getNodeArrayFromChain(targetname)
{
	nodes = [];
	nodes[0] = getnode(targetname, "targetname");
	while (isdefined(nodes[nodes.size - 1].target))
	{
		nodes[nodes.size] = getnode(nodes[nodes.size - 1].target, "targetname");
	}
	return nodes;
}

resetMagicBulletShield(guy)
{
	thread resetMagicBulletShieldThread(guy);
}
resetMagicBulletShieldThread(guy)
{
	guy notify("stop magic bullet shield"); // must stop an earlier one because it might have different properties (like a low health value)
	wait(.01); // wait for it to be stopped
	guy thread maps\_utility::magic_bullet_shield(); // start it anew
}

autoSave(num)
{
	imagename = "levelshots/autosave/autosave_haggerty_" + level.script + num;
	level thread maps\_autosave::healthchecksave(num, &"MYDAWNVILLE_AUTOSAVE_NAME", imagename);
}

eventOnDeath(guy, event)
{
	guy waittill("death");
	eventOccur(event);
}

getRandomAllyNearPlayer(notintrigger)
{
	allies = getaiarray("allies");
	closesti = -1;
	closestdist = 0;
	for (i = 0; i < allies.size; i++)
	{
		if (allies[i] == level.foley || allies[i] == level.baker)
			continue;
		if (isdefined(notintrigger) && allies[i] istouching(notintrigger))
			continue;
		
		dist = distance(level.player.origin, allies[i].origin);
		if (closesti < 0 || dist < closestdist) {
			closesti = i;
			closestdist = dist;
		}
	}
	
	if (closesti == -1)
		return undefined;
	return allies[closesti];
}

exploderChunkFix()
{
	chunks = getentarray("exploderchunk", "targetname");
	num = 0;
	for (i = 0; i < chunks.size; i++)
	{
		if (isdefined(chunks[i].target)) {
			num++;
			org = getent(chunks[i].target, "targetname");
			dir = vectornormalize(org.origin - chunks[i].origin);

			org.origin = chunks[i].origin + maps\_utility::vectorScale(dir, 300 + randomfloat(100));
			//thread myPrint3d(org.origin, "@");
		}
	}
	debug("CHUNKS: " + num);
}

myPrint3d(pos, text)
{
	while(1)
	{
		print3d(pos, text, (1,1,1));
		maps\_spawner::waitframe();
	}
}

exploderLightingFix()
{
	// The shadow brushes surrounding explosions cast shadows on the exploder scriptbrushmodels that remain after the explosion.
	// I don't want this so I moved the exploders away to create separate lighting conditions for them.
	// Now I move them back.
	
	exploders = getentarray("exploder", "targetname");
	for (i = 0; i < exploders.size; i++)
		exploderLightingFixMover(exploders[i]);
	exploders = getentarray("exploderchunk", "targetname");
	for (i = 0; i < exploders.size; i++)
		exploderLightingFixMover(exploders[i]);
}
exploderLightingFixMover(brushmodel)
{
	if (brushmodel.script_exploder == 101 || brushmodel.script_exploder == 102 || brushmodel.script_exploder == 103) {
		brushmodel.origin = brushmodel.origin + (0, 3072, -256);
		if (isdefined(brushmodel.target)) {
			target = getent(brushmodel.target, "targetname");
			target.origin = target.origin + (0, 3072, -256);
		}
	}
}

doRetreat(node)
{
	self endon("death");
	
	prevsuppressionwait = self.suppressionwait;
	prevbravery = self.bravery;
	
	self.suppressionwait = 0;
	self.bravery = 0;
	self setGoalNode(node);
	self.goalradius = 4;
	
	self waittill("goal");
	
	self.suppressionwait = prevsuppressionwait;
	self.bravery = prevbravery;
}

killOffSlowly(guys, minInterval, maxInterval)
{
	guys = maps\_utility::array_randomize(guys);
	for (i = 0; i < guys.size; i++)
	{
		wait(minInterval + randomfloat(maxInterval - minInterval));
		for (; i < guys.size; i++) {
			if (isalive(guys[i]))
				break;
		}
		if (i < guys.size) {
			guys[i] DoDamage(guys[i].health + 50, (randomint(5000)-2500,randomint(5000)-2500,randomint(5000)-2500));
		}
	}
}

// *****************************************************************
// Event handling functions - all designed to be used with one-time events only!
initEvents()
{
	level.events = [];
}
eventOccur(event) // to be called when an event happens
{
	level.events[event] = true;
	level notify (event);
}
waitEvent(event) // waits for an event to happen (checks if it has already happened)
{
	if (isdefined(level.events[event]))
		return;
	level waittill(event);
}
undoEvent(event)
{
	level.events[event] = undefined;
}

eventOnOtherEvent(eventcause, eventeffect)
{
	thread eventOnOtherEventThread(eventcause, eventeffect);
}
eventOnOtherEventThread(eventcause, eventeffect)
{
	if (!isdefined(eventcause) || !isdefined(eventeffect))
		return;
	
	if (isdefined(level.events[eventeffect]))
		return;
	level endon(eventeffect);
	
	waitEvent(eventcause);
	
	eventOccur(eventeffect);
}

waitAndNotify(event) // used with waitAny()
{
	self endon("not needed");
	waitEvent(event);
	self notify("evented");
}
waitAnyEvent(event1, event2, event3, event4) // waits for any event to happen
{
	waitobj = spawnStruct();
	
	if (isdefined(event1)) {
		if (isdefined(level.events[event1]))
			return;
		waitobj thread waitAndNotify(event1);
	}
	if (isdefined(event2)) {
		if (isdefined(level.events[event2])) {
			waitobj notify("not needed");
			return;
		}
		waitobj thread waitAndNotify(event2);
	}
	if (isdefined(event3)) {
		if (isdefined(level.events[event3])) {
			waitobj notify("not needed");
			return;
		}
		waitobj thread waitAndNotify(event3);
	}
	if (isdefined(event4)) {
		if (isdefined(level.events[event4])) {
			waitobj notify("not needed");
			return;
		}
		waitobj thread waitAndNotify(event4);
	}
	
	waitobj waittill("evented");
	waitobj notify("not needed");
}
waitAllEvents(event1, event2, event3, event4) // waits for all events in array to happen
{
	if (isdefined(event1))
		waitEvent(event1);
	if (isdefined(event2))
		waitEvent(event2);
	if (isdefined(event3))
		waitEvent(event3);
	if (isdefined(event4))
		waitEvent(event4);
}

hasEventOccured(event)
{
	return isdefined(level.events[event]);
}
